package xmlcaviargui;

import java.io.*;
import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;

import org.w3c.dom.*;
import javax.xml.parsers.*;

/** 
 * Class solely used to represent the entire data that has been annotated on
 * a sequence of frames in a video (jpg) sequence.  
 *  
 * @author Donald Nairn, rewritten by Paul Clark
 * @version 2.0
 */

public class AllData {
    
    /**
     * An array of 3000 FrameData objects, based upon the assumption that
     * there will be no more than 3000 frames in a given sequence of jpg
     * files
     */
    public FrameData[] frameData;

    /** Variable used to determine the default number of frames
     * Must be altered if the video sequence is longer than 3000
     * frames
     */
    public static final int FRAMES = 4000;

    private File file;

    private PrintWriter writer;

    private BufferedReader reader;

    private String seqName;

    /** 
     * Constructor, makes a new AllData object by creating a new FrameData[FRAMES] array
     */
    public AllData() {
	// array full of nulls so can be determined by ImagePanel whether or not something
	// has been drawn
	frameData = new FrameData[FRAMES];
    }
	
    //PTC method used by writeTo
    private void writeAttribute(Document document, Element object, String element, String value){ 

	object.appendChild(document.createTextNode("    "));

	Element attribute = (Element) document.createElement(element);
	attribute.appendChild(document.createTextNode(value));
	object.appendChild(attribute);
	object.appendChild(document.createTextNode("\n            "));
    }
    
    //PTC method used by writeTo
    private void writetoHypothesis(Document document, Element object, String element, String value){ 

	object.appendChild(document.createTextNode("                        "));

	Element attribute = (Element) document.createElement(element);
	attribute.appendChild(document.createTextNode(value));
	object.appendChild(attribute);
	object.appendChild(document.createTextNode("\n            "));
    }


    //PTC method used by writeTo
    private void writeAttribute(Document document, Element object, String element, String value, 
	                        double evaluation){

	object.appendChild(document.createTextNode("    "));

	Element attribute = (Element) document.createElement(element);
	attribute.appendChild(document.createTextNode(value));
	attribute.setAttribute("evaluation", Double.toString(evaluation));
	object.appendChild(attribute);
	object.appendChild(document.createTextNode("\n            "));
    }

    //PTC method used by writeTo
    private void writetoHypothesis(Document document, Element object, String element, String value, 
	                        double evaluation){

	object.appendChild(document.createTextNode("    "));

	Element attribute = (Element) document.createElement(element);
	attribute.appendChild(document.createTextNode(value));
	attribute.setAttribute("evaluation", Double.toString(evaluation));
	object.appendChild(attribute);
	object.appendChild(document.createTextNode("\n                    "));
    }
//	writetoHypothesis(document, hypothesis, "movement", "none",1);
    //PTC method used by writeTo
    private void writetoBody(Document document,Element father, Element son, Circle circle, boolean visible, double evaluation){
		son.setAttribute("xc", Integer.toString(circle.getX()));	
		son.setAttribute("yc", Integer.toString(circle.getY()));
		son.setAttribute("size", Integer.toString(circle.getR()));	
		son.setAttribute("evaluation", Integer.toString(1));
		if (visible==true)
			son.setAttribute("occluded","no");
		else
			son.setAttribute("occluded","yes");																			
		father.appendChild(document.createTextNode("                         "));
		father.appendChild(son);
		father.appendChild(document.createTextNode("\n"));		
    }    
    
	private void writetoBody(Document document,Element father, Element son,int x, int y, boolean visible, double evaluation){
		son.setAttribute("xc", Integer.toString(x));	
		son.setAttribute("yc", Integer.toString(y));
		son.setAttribute("evaluation", Integer.toString(1));
		if (visible==true)
			son.setAttribute("occluded","no");
		else
			son.setAttribute("occluded","yes");																			
		father.appendChild(document.createTextNode("                         "));
		father.appendChild(son);
		father.appendChild(document.createTextNode("\n"));		
	}      
    
    
    /** 
     * Method used when writing given data to a file so it may be restored
     * and annotation continued
     * @author Paul Clark 5/8/2004
     */
    public void writeTo(String seqName, int id, String fileName) {

	if(!seqName.equals("")) seqName = seqName + "/";
	if (fileName.length() < 4) {
	    fileName = seqName+fileName+".xml";
	}
	else if (!(fileName.substring(fileName.length()-4,fileName.length()).equals(".xml"))) {
	    fileName = seqName+fileName+".xml";
	}
	else fileName = seqName+fileName;

	if(seqName.equals("")) seqName = this.seqName;

	Document document;
	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	SingleBox[] sBoxes;
	GroupBox[] gBoxes;
	boolean isFirstObj;
	boolean isFirstAtt;
	boolean[] sTruths;
	boolean[] gTruths;

	try {
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    document = builder.newDocument();
		String fName=seqName;
//		System.out.println("SeqName is "+fName);
		if (fileName.charAt(fName.length()-1)=='/')
			fName=fName.substring(0,fName.length()-1);
//		System.out.println("SeqName now is "+fName);		
	    Element root = (Element) document.createElement("dataset");
	    root.setAttribute("name", fName);
	    document.appendChild(root);
	    root.appendChild(document.createTextNode("\n\n"));
	    for(int i = 0; i < frameData.length; i++){
		if(frameData[i] == null) continue;
		Element frame = (Element) document.createElement("frame");
		frame.setAttribute("number", Integer.toString(i));
		root.appendChild(document.createTextNode("    "));
		root.appendChild(frame);
		root.appendChild(document.createTextNode("\n\n"));
		
		sBoxes = frameData[i].singleBox;
		Element objectList = null;
		for(int j = 0; j < sBoxes.length; j++){
		    if(sBoxes[j] != null){
			objectList = (Element) document.createElement("objectlist");
			frame.appendChild(document.createTextNode("\n\n        "));
			frame.appendChild(objectList);
			frame.appendChild(document.createTextNode("\n\n    "));
			break;
		    }
		}
		
		isFirstObj = true;
		for(int j = 0; j < sBoxes.length; j++){
		    if(sBoxes[j].isDefault()) continue;
		    if(isFirstObj){
			objectList.appendChild(document.createTextNode("\n\n            "));
		        isFirstObj = false;
		    }
		    else 
			objectList.appendChild(document.createTextNode("    "));
		    Element object = (Element) document.createElement("object");
		    object.setAttribute("id", Integer.toString(j));
		    objectList.appendChild(object);
		    objectList.appendChild(document.createTextNode("\n\n        "));
		    

		    //<orientation>
		    object.appendChild(document.createTextNode("\n                "));
		    Element orientation = (Element) document.createElement("orientation");
		    orientation.appendChild(document.createTextNode(
						Integer.toString(sBoxes[j].getAxis())));
		    object.appendChild(orientation);
		    object.appendChild(document.createTextNode("\n            "));
		    //</orientaton>
	    
		    //<box>
		    Rectangle rect = sBoxes[j].getBox();
		    if(rect != null){
		    object.appendChild(document.createTextNode("    "));
		    Element box = (Element) document.createElement("box");
		    int w = (int) rect.getWidth();
		    int h = (int) rect.getHeight();
		    int x = (int) rect.getX() + w/2;
		    int y = (int) rect.getY() + h/2;
		    box.setAttribute("xc", Integer.toString(x));
		    box.setAttribute("yc", Integer.toString(y));		    
		    box.setAttribute("w", Integer.toString(w));
		    box.setAttribute("h", Integer.toString(h));
		    object.appendChild(box);
		    object.appendChild(document.createTextNode("\n            "));
		    }
		    //</box>

			//<body>
			Body body = sBoxes[j].getBody();
			Element bodyList=null;
			if (body.isNull()==false){
				bodyList = (Element) document.createElement("body");
				bodyList.appendChild(document.createTextNode("\n                "));
				if (body.isHeadDraw()==true){
					 Element headelement=(Element) document.createElement("head"); 			    	
					Circle head=body.getHead();
//					System.out.println("The head for j "+j+" is xc "+head.getX()+" "+head.getY()+" r "+head.getR());
					headelement.setAttribute("xc", Integer.toString(head.getX()));	
					headelement.setAttribute("yc", Integer.toString(head.getY()));
					headelement.setAttribute("size", Integer.toString(head.getR()));	
					headelement.setAttribute("gaze", Integer.toString(body.getGaze()));						
					headelement.setAttribute("evaluation", Integer.toString(1));	
					if (body.isVisiblehead()==true)
						headelement.setAttribute("occluded","no");
					else
						headelement.setAttribute("occluded","yes");																		
					bodyList.appendChild(document.createTextNode("    "));	
					bodyList.appendChild(headelement);		    	
					bodyList.appendChild(document.createTextNode("\n                "));	
				}
				
				if (body.isShouldersDraw()==true){
					Element shouldersEl=(Element) document.createElement("shoulders"); 	
					shouldersEl.appendChild(document.createTextNode("\n"));		    	
					if (body.isShouldersDraw()==true){
						Line2D.Float sh=body.getShoulderLine();
						Element right=(Element) document.createElement("right"); 
						writetoBody(document,shouldersEl,right,(int)sh.getX1(),(int)sh.getY1(),body.isVisiblerightshoulder(),1.0);
						Element left=(Element) document.createElement("left"); 
						writetoBody(document,shouldersEl,left,(int)sh.getX2(),(int)sh.getY2(),body.isVisibleleftshoulder(),1.0);
//						System.out.println("The left foot for j "+j+" is xc "+rightfoot.getX()+" "
//											+rightfoot.getY()+" r "+rightfoot.getR());
											
					}					
					shouldersEl.appendChild(document.createTextNode("                    "));	
					bodyList.appendChild(document.createTextNode("    "));	
					bodyList.appendChild(shouldersEl);		    	
					bodyList.appendChild(document.createTextNode("\n                "));			
				}				
				if ((body.isLeftHandDraw()==true) || (body.isRightHandDraw()==true)){
					Element handsEl=(Element) document.createElement("hands"); 	
					handsEl.appendChild(document.createTextNode("\n"));		    	
					if (body.isRightHandDraw()==true){
						Circle righthand=body.getRighthand();
						Element right=(Element) document.createElement("right"); 
						writetoBody(document,handsEl,right,righthand,body.isVisiblerighthand(),1.0);
//						System.out.println("The right hand for j "+j+" is xc "+righthand.getX()+" "
//											+righthand.getY()+" r "+righthand.getR());
											
					}	
					if (body.isLeftHandDraw()==true){
						Circle lefthand=body.getLefthand();
						Element left=(Element) document.createElement("left"); 
						writetoBody(document,handsEl,left,lefthand,body.isVisiblelefthand(),1.0);
//						System.out.println("The left hand for j "+j+" is xc "+lefthand.getX()+" "
//											+lefthand.getY()+" r "+lefthand.getR());
											
					}				
					handsEl.appendChild(document.createTextNode("                    "));	
					bodyList.appendChild(document.createTextNode("    "));	
					bodyList.appendChild(handsEl);		    	
					bodyList.appendChild(document.createTextNode("\n                "));			
				}
				if ((body.isLeftFootDraw()==true) || (body.isRightFootDraw()==true)){
					Element footEl=(Element) document.createElement("feet"); 	
					footEl.appendChild(document.createTextNode("\n"));		    	
					if (body.isRightFootDraw()==true){
						Circle rightfoot=body.getRightfoot();
						Element right=(Element) document.createElement("right"); 
						writetoBody(document,footEl,right,rightfoot,body.isVisiblerightfoot(),1.0);
//						System.out.println("The left foot for j "+j+" is xc "+rightfoot.getX()+" "
//											+rightfoot.getY()+" r "+rightfoot.getR());
											
					}
					if (body.isLeftFootDraw()==true){
						Circle leftfoot=body.getLeftfoot();
						Element left=(Element) document.createElement("left"); 
						writetoBody(document,footEl,left,leftfoot,body.isVisibleleftfoot(),1.0);
//						System.out.println("The left foot for j "+j+" is xc "+leftfoot.getX()+" "
//											+leftfoot.getY()+" r "+leftfoot.getR());
											
					}					
					footEl.appendChild(document.createTextNode("                    "));	
					bodyList.appendChild(document.createTextNode("    "));	
					bodyList.appendChild(footEl);		    	
					bodyList.appendChild(document.createTextNode("\n                "));			
				}
				object.appendChild(document.createTextNode("    "));
				object.appendChild(bodyList);
				object.appendChild(document.createTextNode("\n            "));	


			}
					    
		    sTruths = sBoxes[j].truths;
		    isFirstAtt = true;
			Element hypothesislist=null;
			Element hypothesis=null;
			
			hypothesis=(Element) document.createElement("hypothesis"); 
		    hypothesis.setAttribute("id","1");
  		    if(sTruths[SingleBox.APPEAR]==false)
		    		hypothesis.setAttribute("prev","1.0");  
		    else
		    		hypothesis.setAttribute("prev","0.0"); 		    
		    hypothesis.setAttribute("evaluation","1.0");
		    hypothesis.appendChild(document.createTextNode("\n"));
		    
			 hypothesislist = (Element) document.createElement("hypothesislist");
		    hypothesislist.appendChild(document.createTextNode("\n                    "));
			 hypothesislist.appendChild(hypothesis);
		    hypothesislist.appendChild(document.createTextNode("\n                 "));

			 
		    //<appearance>
		    if(sTruths[SingleBox.APPEAR]){
			writeAttribute(document, object, "appearance", "appear");
			 object.appendChild(document.createTextNode("    "));
			 object.appendChild(hypothesislist);	
			 object.appendChild(document.createTextNode("\n            "));			 		
			 isFirstAtt = false;
		    }
		    if(sTruths[SingleBox.DISAPPEAR]){
			writeAttribute(document, object, "appearance", "disappear");
			 object.appendChild(document.createTextNode("    "));
			 object.appendChild(hypothesislist);
			 object.appendChild(document.createTextNode("\n            "));			
			isFirstAtt = false;
		    }
		    if(sTruths[SingleBox.OCCLUDED]){
			writeAttribute(document, object, "appearance", "occluded");
			 object.appendChild(document.createTextNode("    "));
			 object.appendChild(hypothesislist);		
			 object.appendChild(document.createTextNode("\n            "));			 
			isFirstAtt = false;
		    }
		    //</appearance>
		    
		    //<movement>
		    if(sTruths[SingleBox.WALK]){
			if(isFirstAtt){
			    writeAttribute(document, object, "appearance","visible");
				 object.appendChild(document.createTextNode("    "));
				 object.appendChild(hypothesislist);			
			 	 object.appendChild(document.createTextNode("\n            "));				 	
			    isFirstAtt = false;
			}
			hypothesis.appendChild(document.createTextNode("                    "));
			writetoHypothesis(document, hypothesis, "movement", "walking",1);
		    }
		    if(sTruths[SingleBox.RUN]){
			if(isFirstAtt){
			    writeAttribute(document, object, "appearance","visible");
				 object.appendChild(document.createTextNode("    "));
				 object.appendChild(hypothesislist);	
			 	object.appendChild(document.createTextNode("\n            "));				 			
			    isFirstAtt = false;
			}
			hypothesis.appendChild(document.createTextNode("                    "));			
			writetoHypothesis(document,hypothesis, "movement", "running",1);
		    }
		    if(sTruths[SingleBox.ACTIVE]){
			if(isFirstAtt){
			    writeAttribute(document, object, "appearance","visible");
				 object.appendChild(document.createTextNode("    "));
				 object.appendChild(hypothesislist);
			 	 object.appendChild(document.createTextNode("\n            "));				 				
			    isFirstAtt = false;
			}
			hypothesis.appendChild(document.createTextNode("                    "));			
			writetoHypothesis(document, hypothesis, "movement", "active",1);
		    }
		    if(sTruths[SingleBox.INACTIVE]){
			if(isFirstAtt){
			    writeAttribute(document, object, "appearance","visible");
				 object.appendChild(document.createTextNode("    "));
				 object.appendChild(hypothesislist);				
			 	 object.appendChild(document.createTextNode("\n            "));				 
			    isFirstAtt = false;
			}
			hypothesis.appendChild(document.createTextNode("                    "));			
			writetoHypothesis(document, hypothesis, "movement", "inactive",1);
		    }
		    if(sTruths[SingleBox.NONE_MOV]){
			if(isFirstAtt){
			    writeAttribute(document, object, "appearance","visible");
				 object.appendChild(document.createTextNode("    "));
				 object.appendChild(hypothesislist);			
			 	 object.appendChild(document.createTextNode("\n            "));				 
			    isFirstAtt = false;
			}
			hypothesis.appendChild(document.createTextNode("                    "));			
			writetoHypothesis(document, hypothesis, "movement", "none",1);
		    }
		    //</movement>

		    //<role>
		    if(sTruths[SingleBox.FIGHTER]){
			writetoHypothesis(document, hypothesis, "role", "fighter", 1);
		    }
		    if(sTruths[SingleBox.BROWSER]){
			writetoHypothesis(document, hypothesis, "role", "browser",1);
		    }
		    if(sTruths[SingleBox.LEFT_VICTIM]){
			writetoHypothesis(document, hypothesis, "role", "leaving victim", 1); 
		    }
		    if(sTruths[SingleBox.LEAVING_GROUP]){
			writetoHypothesis(document, hypothesis, "role", "leaving group",1);
		    }
		    if(sTruths[SingleBox.WALKER]){
			writetoHypothesis(document,hypothesis, "role", "walker",1);
		    }
		    if(sTruths[SingleBox.LEFT_OBJECT]){
			writetoHypothesis(document,hypothesis, "role", "leaving object",1);
		    }
		    if(sTruths[SingleBox.NONE_ROLE]){
			writetoHypothesis(document, hypothesis, "role", "none",1);
		    }
		    //</role>

		    //event>
		    if(sTruths[SingleBox.PERSON_IMMOBILE]){
			writetoHypothesis(document, hypothesis, "event", "immobile",1);
		    }
		    //</event>

		    //<scenario>
		    if(sTruths[SingleBox.BROWSING]){
			writetoHypothesis(document, hypothesis, "context", "browsing",1);
		    }
		    if(sTruths[SingleBox.IMMOBILE]){
			writetoHypothesis(document, hypothesis, "context", "immobile",1);
		    }
		    if(sTruths[SingleBox.WALKING]){
			writetoHypothesis(document, hypothesis, "context", "walking",1);
		    }
		    if(sTruths[SingleBox.DROP_DOWN]){
			writetoHypothesis(document, hypothesis, "context", "drop down",1);
		    }
		    if(sTruths[SingleBox.WINDOWSHOP_SCE]){
			writetoHypothesis(document, hypothesis, "context", "windowshop",1);
		    }
		    if(sTruths[SingleBox.SHOPENTER_SCE]){
			writetoHypothesis(document, hypothesis, "context", "shop enter",1);
		    }
		    if(sTruths[SingleBox.SHOPREENTER_SCE]){
			writetoHypothesis(document, hypothesis, "context", "shop reenter",1);
			isFirstAtt = false;
		    }
		    if(sTruths[SingleBox.SHOPEXIT_SCE]){
			writetoHypothesis(document, hypothesis, "context", "shop exit",1);
			isFirstAtt = false;
		    }
		    if(sTruths[SingleBox.NONE_SCE]){
			writetoHypothesis(document, hypothesis, "context", "none",1);
			isFirstAtt = false;
		    }
		    //</scenario>

		    //<situation>
		    if(sTruths[SingleBox.MOVING_SIT]){
			writetoHypothesis(document, hypothesis, "situation", "moving",1);
			isFirstAtt = false;
		    }
		    if(sTruths[SingleBox.INACTIVE_SIT]){
			writetoHypothesis(document, hypothesis, "situation", "inactive",1);
			isFirstAtt = false;
		    }
		    if(sTruths[SingleBox.BROWSING_SIT]){
			writetoHypothesis(document, hypothesis,"situation", "browsing",1);
			isFirstAtt = false;
		    }
		    if(sTruths[SingleBox.SHOPENTER_SIT]){
			writetoHypothesis(document, hypothesis,"situation", "shop enter",1);
			isFirstAtt = false;
		    }
		    if(sTruths[SingleBox.SHOPEXIT_SIT]){
			writetoHypothesis(document, hypothesis, "situation", "shop exit",1);
			isFirstAtt = false;
		    }
		    if(sTruths[SingleBox.NONE_SIT]){
			writetoHypothesis(document, hypothesis,"situation", "none",1);
			isFirstAtt = false;
		    }
		    //</situation>
		}

		gBoxes = frameData[i].groupBox;
		Element groupList = null;
		for(int j = 0; j < gBoxes.length; j++){
		    if(gBoxes[j] != null){
			groupList = (Element) document.createElement("grouplist");
			frame.appendChild(document.createTextNode("    "));
			frame.appendChild(groupList);
			frame.appendChild(document.createTextNode("\n\n    "));
			break;
		    }
		}
		
		isFirstObj = true;
		for(int j = 0; j < gBoxes.length; j++){
		    if(gBoxes[j].isDefault()) continue;
		    if(isFirstObj){
			groupList.appendChild(document.createTextNode("\n\n            "));
		        isFirstObj = false;
		    }
		    else 
			groupList.appendChild(document.createTextNode("    "));
		    Element group = (Element) document.createElement("group");
		    group.setAttribute("id", Integer.toString(j));
		    groupList.appendChild(group);
		    groupList.appendChild(document.createTextNode("\n\n        "));

		    //<orientation>
		    group.appendChild(document.createTextNode("\n                "));
		    Element orientation = (Element) document.createElement("orientation");
		    orientation.appendChild(document.createTextNode(
						Integer.toString(gBoxes[j].getAxis())));
		    group.appendChild(orientation);
		    group.appendChild(document.createTextNode("\n            "));
		    //</orientaton>
		    
		    //<box>
		    Rectangle rect = gBoxes[j].getBox();
		    if(rect != null){
		    group.appendChild(document.createTextNode("    "));
		    Element box = (Element) document.createElement("box");
		    int w = (int) rect.getWidth();
		    int h = (int) rect.getHeight();
		    int x = (int) rect.getX() + w/2;
		    int y = (int) rect.getY() + h/2;
		    box.setAttribute("xc", Integer.toString(x));
		    box.setAttribute("yc", Integer.toString(y));
		    box.setAttribute("w", Integer.toString(w));
		    box.setAttribute("h", Integer.toString(h));
		    group.appendChild(box);
		    group.appendChild(document.createTextNode("\n            "));
		    }
		    //</box>

		    //<members>
		    String membs = "";
		    boolean fst = true;
		    for(int k = 0; k < FrameData.SINGLE_BOXES; k++){
			if(gBoxes[j].indBoxes[k]){
			    if(fst){
				membs = Integer.toString(k);
				fst = false;
			    }
			    else
			    	membs = membs + "," + Integer.toString(k);
			}
		    }
		    writeAttribute(document, group, "members", membs);
		    //</members>
		    
		    gTruths = gBoxes[j].truths;
		    isFirstAtt = true;

		    Element ghypothesislist=null;
		    Element ghypothesis=null;
			 

		    ghypothesis=(Element) document.createElement("hypothesis"); 
		    ghypothesis.setAttribute("id","1");
  		    if(gTruths[GroupBox.APPEAR]==false)
		    		ghypothesis.setAttribute("prev","1.0");  
		    else
		    		ghypothesis.setAttribute("prev","0.0");  
		    ghypothesis.setAttribute("evaluation","1.0");
		    ghypothesis.appendChild(document.createTextNode("\n"));
		    
		    ghypothesislist = (Element) document.createElement("hypothesislist");
		    ghypothesislist.appendChild(document.createTextNode("\n                    "));
		    ghypothesislist.appendChild(ghypothesis);
		    ghypothesislist.appendChild(document.createTextNode("\n                 "));

	    
		    //<appearance>
		    if(gTruths[GroupBox.APPEAR]){
			writeAttribute(document, group, "appearance", "appear");
			 group.appendChild(document.createTextNode("    "));
			 group.appendChild(ghypothesislist);
			 group.appendChild(document.createTextNode("\n            "));			 			
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.DISAPPEAR]){
			writeAttribute(document, group, "appearance", "disappear");
			 group.appendChild(document.createTextNode("    "));
			 group.appendChild(ghypothesislist);			
			 group.appendChild(document.createTextNode("\n            "));				
			isFirstAtt = false;
		    }
		    //</appearance>
		    
		    //<movement>
		    if(gTruths[GroupBox.ACTIVE]){
			if(isFirstAtt){
			    writeAttribute(document, group, "appearance", "visible");
			    group.appendChild(document.createTextNode("    "));
			    group.appendChild(ghypothesislist);
			    group.appendChild(document.createTextNode("\n            "));				    
			    isFirstAtt = false;
			}
			ghypothesis.appendChild(document.createTextNode("                    "));			
			writetoHypothesis(document,ghypothesis, "movement", "active",1);
		    }
		    if(gTruths[GroupBox.INACTIVE]){
			if(isFirstAtt){
			    writeAttribute(document, group, "appearance","visible");
			    group.appendChild(document.createTextNode("    "));
			    group.appendChild(ghypothesislist);			    
			    group.appendChild(document.createTextNode("\n            "));				    
			    isFirstAtt = false;
			}
			ghypothesis.appendChild(document.createTextNode("                    "));						
			writetoHypothesis(document, ghypothesis, "movement", "inactive",1);
		    }
		    if(gTruths[GroupBox.MOVE]){
			if(isFirstAtt){
			    writeAttribute(document, group, "appearance", "visible");
			    group.appendChild(document.createTextNode("    "));
			    group.appendChild(ghypothesislist);			    
			    group.appendChild(document.createTextNode("\n            "));				    
			    isFirstAtt = false;
			}
			ghypothesis.appendChild(document.createTextNode("                    "));						
			writetoHypothesis(document, ghypothesis, "movement", "movement",1);
		    }
		    if(gTruths[GroupBox.NONE_MOV]){
			if(isFirstAtt){
			    writeAttribute(document, group, "appearance","visible");
			    group.appendChild(document.createTextNode("    "));
			    group.appendChild(ghypothesislist);			    
			    group.appendChild(document.createTextNode("\n            "));				    
			    isFirstAtt = false;
			}
			ghypothesis.appendChild(document.createTextNode("                    "));			
			writetoHypothesis(document, ghypothesis, "movement", "none",1);
		    }
		    //</movement>

		    //<role>
		    if(gTruths[GroupBox.FIGHTERS]){
			writetoHypothesis(document, ghypothesis,"role", "fighters",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.MEETERS]){
			writetoHypothesis(document, ghypothesis,"role", "meeters",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.WALKERS]){
			writetoHypothesis(document, ghypothesis, "role", "walkers",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GRBROWSER_ROLE]){
			writetoHypothesis(document, ghypothesis,"role", "browsers",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GRLEFTVICTIM_ROLE]){
			writetoHypothesis(document, ghypothesis,"role", "leaving victim",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GRLEFTOBJECT_ROLE]){
			writetoHypothesis(document, ghypothesis,"role", "leaving object",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.NONE_ROLE]){
			writetoHypothesis(document, ghypothesis, "role", "none",1);
			isFirstAtt = false;
		    }
		    //</role>

		    //<event>
		    if(gTruths[GroupBox.FIGHT_START]){
			writetoHypothesis(document, ghypothesis, "event", "fight start",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.FIGHT_END]){
			writetoHypothesis(document, ghypothesis, "event", "fight end",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.FIGHT_VICTIM_LEFT]){
			writetoHypothesis(document, ghypothesis, "event", "left victim",1);
			isFirstAtt = false;
		    }
		    //</event>

		    //<scenario>
		    if(gTruths[GroupBox.FIGHTING]){
			writetoHypothesis(document, ghypothesis, "context", "fighting",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.MEETING]){
			writetoHypothesis(document, ghypothesis,"context", "meeting",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.LEAVING]){
			writetoHypothesis(document, ghypothesis,"context", "leaving",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GRWINDOWSHOP_SCE]){
			writetoHypothesis(document, ghypothesis, "context", "windowshop",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GRSHOPENTER_SCE]){
			writetoHypothesis(document, ghypothesis, "context", "shop enter",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GRSHOPEXIT_SCE]){
			writetoHypothesis(document, ghypothesis, "context", "shop exit",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GRSHOPREENTER_SCE]){
			writetoHypothesis(document, ghypothesis, "context", "shop reenter",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GRWALKING_SCE]){
			writetoHypothesis(document, ghypothesis, "context", "walking",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.NONE_SCE]){
			writetoHypothesis(document, ghypothesis, "context", "none",1);
			isFirstAtt = false;
		    }
		    //</scenario>

		    //<situation>
		    if(gTruths[GroupBox.GR_FIGHTING]){
			writetoHypothesis(document, ghypothesis, "situation", "fighting",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GR_MOVING]){
			writetoHypothesis(document, ghypothesis, "situation", "moving",	1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GR_MEETING]){
			writetoHypothesis(document, ghypothesis, "situation", "joining",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GR_SPLITTING]){
			writetoHypothesis(document, ghypothesis,"situation", "split up",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GR_INACTIVE]){
			writetoHypothesis(document, ghypothesis,"situation", "inactive",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GR_LEFT_VICTIM]){
			writetoHypothesis(document, ghypothesis, "situation", "leaving victim",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GR_LEFT_OBJECT]){
			writetoHypothesis(document, ghypothesis,"situation", "leaving object",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GR_INTERACTING]){
			writetoHypothesis(document, ghypothesis,"situation", "interacting",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GRSHOPENTER_SIT]){
			writetoHypothesis(document, ghypothesis, "situation", "shop enter",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GRSHOPEXIT_SIT]){
			writetoHypothesis(document, ghypothesis,"situation", "shop exit",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.GRBROWSING_SIT]){
			writetoHypothesis(document, ghypothesis, "situation", "browsing",1);
			isFirstAtt = false;
		    }
		    if(gTruths[GroupBox.NONE_SIT]){
			writetoHypothesis(document, ghypothesis,"situation", "none",1);
			isFirstAtt = false;
		    }
		    //</situation>
		}
	    }
	    DocumentWriter w = new DocumentWriter();
 	    w.write(document, fileName);
	}

	catch(ParserConfigurationException e) {
		JOptionPane.showMessageDialog(null,
					      ("Could not write file:\n "+fileName+"\nto Disk. Check file permissions"),
					      "Parse Error",
					      JOptionPane.WARNING_MESSAGE);
	    
	}
	catch(IOException e){
		JOptionPane.showMessageDialog(null,
					      ("Could not write file:\n "+fileName+"\nTo Disk"),
					      "Write Error",
					      JOptionPane.WARNING_MESSAGE);
	    
	}
    }
    
    //PTC removed
    /*public void writeTo(String seqName, int id, String fileName) {
       
	if (fileName.length() < 4) {
	    fileName = seqName+"/"+fileName+".txt";
	} else if (!(fileName.substring(fileName.length()-4,fileName.length()).equals(".txt"))) {
	    fileName = seqName+"/"+fileName+".txt";
	} else {
	    fileName = seqName+"/"+fileName;
	}
	
	file = new File(fileName);

	if (!file.exists()) {
	    try {
		file.createNewFile();
	    } catch (IOException ioe) {
		JOptionPane.showMessageDialog(null,
					      ("Could not write file:\n "+fileName+"\nTo Disk"),
					      "Write Error",
					      JOptionPane.WARNING_MESSAGE);
	    }
	}

	try {
	    writer = new PrintWriter(new FileWriter(file));
	    writeAll(seqName);
	} catch (IOException ioe) {
	    JOptionPane.showMessageDialog(null,
					  ("Could not create file writer for:\n"+fileName),
					  "Write Error",
					  JOptionPane.WARNING_MESSAGE);
	}
	
    }*/

    /* 
     * Method used to write all of the data from sequence name
     * seqName to disk
     * PTC - now obsolete
     */
    /*public void writeAll(String seqName) {

	for (int i = 0; i < frameData.length; i++) {
	    if (frameData[i] != null) {
		String writeString = writeString(i,seqName,frameData[i]);
		writer.println(writeString);
	    } else { continue; }
	}
	writer.close();
	
    }*/

    /*
     * Method used to return the data gathered from frame number
     * id by the user's annotations
     *PTC - now obsolete
     */
    /*public String writeString(int id, String seqName, FrameData data) {
	// header
	String writeString = "frame\t"+seqName+"\t"+id+"\t";
	// convert frameData to its string representation
	
	writeString = writeString + data.toString();
	
	// end of frame
	writeString = writeString + "eframe\n";
	
	return writeString;
    }*/

    /** 
     * Method used to read data from disk stored in a .xml file,
     * returns the string consisting of all the data in text file
     * fileName
     * @author Paul Clark 5/8/04
     */
     
    public String readFrom(String seqName, String fileName){

	DocumentReader r = new DocumentReader();
	Document doc = null;
	try{
	    if(seqName.equals(""))
		doc = r.read(fileName);
	    else
		doc = r.read(seqName+"/"+fileName);
	    System.out.println("READING FROM  HERE "+seqName+"/"+fileName);
	    return updateData(doc);
	}
	catch(FileNotFoundException e){
	    JOptionPane.showMessageDialog(null,
                                          ("Error reading file "+fileName+"\n"+
                                           "from directory "+seqName+"\nFile does not exist"),
                                           "Read Error",
                                           JOptionPane.WARNING_MESSAGE);
            return "";
	}
	catch(IOException e){
	    JOptionPane.showMessageDialog(null,
   				          ("Error reading file "+fileName+"\n"+
				           "from  directory "+seqName+"\n"),
			   	           "Read Error",
				           JOptionPane.WARNING_MESSAGE);
	    return "";
	}
   }

    //PTC removed above used instead
    /*public String  readFrom(String seqName, String fileName) {
	
	PTC String returnString = "";
	Object [] data = new Object []{""};
	file = new File(seqName+"/"+fileName);
	   
	if (!file.exists()) {
	    file = new File(seqName+"/"+fileName+".txt");
	    if (!file.exists()) {
		JOptionPane.showMessageDialog(null,
					      ("Error reading file "+fileName+"\n"+
					       "from directory "+seqName+"\nFile does not exist"),
					      "Read Error",
					      JOptionPane.WARNING_MESSAGE);
		return "";
	    }
	} 
	if (file.exists()) {

	    try {
		reader = new BufferedReader(new FileReader(file));
	    } catch (IOException ioe) {
		JOptionPane.showMessageDialog(null,
					      ("Error reading file "+fileName+"\n"+
					       "from  directory "+seqName+"\nFile does not exist"),
					      "Read Error",
					      JOptionPane.WARNING_MESSAGE);
		return "";
	    }
		//HUGO
		int cou=0;
		LinkedList readString = new LinkedList();
		String temp, tempArray[];
		try {
		String line;
		System.out.println("Frame data read so far:");				
		while ((line = reader.readLine()) != null) {
			temp = line.trim();
			tempArray = temp.split("\t");
			for(int i=0;i<tempArray.length;i++)
				readString.add(tempArray[i]);
			cou++;
			if(cou%100 == 0)
				System.out.print(" " + cou/2);
		}
		data = readString.toArray();
		System.out.println("\nDATA FROM FILE READ");
		//ENDHUGO	
	
	    } catch (IOException ioe) {
		JOptionPane.showMessageDialog(null,
					      ("Error reading file "+fileName),
					      "Read Error",
					      JOptionPane.WARNING_MESSAGE);
		return "";
	    }
	    	    
	    // reset data structure
	    frameData = new FrameData[FRAMES];

	    returnString = updateData(data);
	    try {
		reader.close();
	    } catch (IOException ioe) {
		JOptionPane.showMessageDialog(null,
					      ("Could not close file "+fileName),
					      "Read Error",
					      JOptionPane.WARNING_MESSAGE);
		return "";
	    }	
	} 
	//System.out.println(returnString);
	System.out.println("\n\nDONE!!!");
	return returnString;
    }*/

    /* PTC convenience method */
    private double getEval(Node n, int fid){
	String s = ((Element) n).getAttribute("evaluation");
	if(s.equals("")){
	    JOptionPane.showMessageDialog(null,
		                        ("Bad number format in frame "+fid),	                      
	      				"Number Error",
					JOptionPane.WARNING_MESSAGE);
	    return -1;
	}
	try{
	    return Double.parseDouble(s);
	}catch(NumberFormatException e){
	    JOptionPane.showMessageDialog(null,
		                        ("Bad number format in frame "+fid),	                      
	      				"Number Error",
					JOptionPane.WARNING_MESSAGE);
	    return -1;
	}
    }

    /**
	      * Method used by readFrom to read in the xml file and update
	      * all of the necessary datastructures, adding the new structure that includes the body tag
	      * @author Jose Vazquez 21/may/05
	      */
	    public String updateData(Document doc){
		int framenumber=0;
		int highest = 0;
	    try{
	//    	System.out.println("starting to load the data");
		Node frame;
		Node list;
		Node element;
		Node attribute;
		SingleBox[] sBoxes = new SingleBox[FrameData.SINGLE_BOXES];
		GroupBox[] gBoxes = new GroupBox[FrameData.GROUP_BOXES];
		boolean[] sTruths = new boolean[SingleBox.LENGTH];
		boolean[] gTruths = new boolean[GroupBox.LENGTH];
		boolean[] gIndBoxes = new boolean[FrameData.SINGLE_BOXES];
		boolean lgwarned = false;
		frameData = new FrameData[FRAMES];
		int axis = 90;
		Rectangle box = new Rectangle();

	
		Line2D.Float line = new Line2D.Float();
		int fid;
	
		for (int i = 0; i < gBoxes.length; i++) {
		    gBoxes[i] = new GroupBox();
		}
	
		for (int i = 0; i < sBoxes.length; i++) {
		    sBoxes[i] = new SingleBox();
		}
		
	
		frame = doc.getDocumentElement().getFirstChild();
		
		//<frame>
		for(;frame != null; frame = frame.getNextSibling()){
	
		    if(!frame.getNodeName().equals("frame")) continue;
		    
		    String fidString = ((Element) frame).getAttribute("number");
		    if(fidString.equals("")){
		    	JOptionPane.showMessageDialog(null,
	   	        	  ("Invalid frame id " + fidString),
		             	   "Number Error",
		           	   JOptionPane.WARNING_MESSAGE);
				   return "";
		    
		    }
		    try{
		    	fid = Integer.parseInt(fidString);
		    }catch(NumberFormatException e){
		    	JOptionPane.showMessageDialog(null,
	   	        	  ("Invalid frame id " + fidString),
		             	   "Number Error",
		           	   JOptionPane.WARNING_MESSAGE);
				   return "";
	
		    }
	
		    for(int i = 0; i< sBoxes.length; i++)
			sBoxes[i] = new SingleBox();
		    for(int i = 0; i< gBoxes.length; i++)
			gBoxes[i] = new GroupBox();
			    
			    
		    //<objectlist> , <grouplist>
			framenumber++;
		    for(list = frame.getFirstChild(); 
			    list != null; list = list.getNextSibling()){
	
			//<objectlist>
			if(list.getNodeName().equals("objectlist")){
	//			System.out.println("accesing list of elements");
			    //<object>
			    for(element = list.getFirstChild();element != null; element = element.getNextSibling()){
	
				if(!element.getNodeName().equals("object")) continue;
				
				// reset all attributes
				for (int j = 0; j < sTruths.length; j++) {
				    sTruths[j] = false;
				}
				sTruths[SingleBox.NONE_ROLE] = sTruths[SingleBox.NONE_SIT] =
				    sTruths[SingleBox.NONE_SCE] = sTruths[SingleBox.NONE_MOV] = true;
				
				box = null;
				Body body = new Body();
				axis = 90;
	
				//<orientation>, <box>, <appearance>, <movement>, <role>
				//<scenario>, <situation>, <event>
				boolean foundBox = false;
				boolean foundOrientation = false;
				for(attribute = element.getFirstChild(); 
					attribute != null; attribute = attribute.getNextSibling()){
				    
				    //<orientation>
				    if(attribute.getNodeName().equals("orientation")){
				       Node n;
				       for(n = attribute.getFirstChild();
					       !n.getNodeName().equals("#text") && n != null; 
					       n = n.getNextSibling());
				       
				       String axisString = n.getNodeValue();
					try{ 
					    axis = Integer.parseInt(axisString);
					}catch(NumberFormatException e){
		    				JOptionPane.showMessageDialog(null,
	   				        	  (axisString + 
							   " not a valid orientation in frame " 
							   + fid),
				   	           	   "Number Error",
					           	   JOptionPane.WARNING_MESSAGE);
						return "";
					}
					foundOrientation = true;
				    }//</orientation>
	
				    //<box>
				    if(attribute.getNodeName().equals("box")){
					try{
					    String w = ((Element) attribute).getAttribute("w");
					    String h = ((Element) attribute).getAttribute("h");
					    String x = ((Element) attribute).getAttribute("x");
					    String y = ((Element) attribute).getAttribute("y");
					    String xc= ((Element) attribute).getAttribute("xc");
					    String yc= ((Element) attribute).getAttribute("yc");
					    boolean nocenter=((x.equals("") || y.equals("")) && 
					                      (xc.equals("") || yc.equals("")));
	               
					    boolean center=(xc.equals("") || yc.equals(""));
	//				    System.out.println("no center its  "+nocenter+" center "+center);  
					    if(w.equals("") || h.equals("") || nocenter){
		    				JOptionPane.showMessageDialog(null,
	   				        	  ("Invalid box dimensions in frame " + fid),
				   	           	   "Number Error",
					           	   JOptionPane.WARNING_MESSAGE);
						return "";
					    }
					    int wi = Integer.parseInt(w);
					    int hi = Integer.parseInt(h);
					    int xi=0,yi=0;
					    if (center==true){
						    xi = Integer.parseInt(x);
						    yi = Integer.parseInt(y);
						 }else{
						 	 xi= Integer.parseInt(xc) - wi/2;
							 yi= Integer.parseInt(yc) - hi/2;
						 }
					    box = new Rectangle(xi,yi,wi,hi);
					}
					catch(NumberFormatException e){ 
		    				JOptionPane.showMessageDialog(null,
	   				        	  ("Invalid box dimensions in frame " + fid),
				   	           	   "Number Error",
					           	   JOptionPane.WARNING_MESSAGE);
						return "";
					}
					foundBox = true;
				    }//</box>
				    //<body>
				    if(attribute.getNodeName().equals("body")){
					       Node bodyXML;
						    for(bodyXML = attribute.getFirstChild();bodyXML!= null ;
						    	bodyXML = bodyXML.getNextSibling()){
						    	if (bodyXML.getNodeName().equals("head")){
						    		try{
								    String xc_str = ((Element) bodyXML).getAttribute("xc");
								    String yc_str = ((Element) bodyXML).getAttribute("yc");
								    String radious_str = ((Element) bodyXML).getAttribute("size");
								    String gaze_str = ((Element) bodyXML).getAttribute("gaze");
								    String state_str = ((Element) bodyXML).getAttribute("occluded");
								    int xc=Integer.parseInt(xc_str);
								    int yc=Integer.parseInt(yc_str);
								    int radious=Integer.parseInt(radious_str);
								    int gaze=Integer.parseInt(gaze_str);
								    if (state_str.equals("no"))
								    	body.setVisiblehead(true);
								    else
								    	body.setVisiblehead(false);
//								    System.out.println("head xc "+xc+" "+yc+" size "+radious+" gaze "+gaze+" state "+body.isVisiblehead());    
								    Circle head=new Circle(xc,yc,radious,Circle.HEAD);
								    body.setGaze(gaze);
								    body.setHead(head);
						    		}catch(Exception e){
						    			e.printStackTrace();
					    				JOptionPane.showMessageDialog(null,
					   				        	  ("Invalid head dimensions in frame " + fid),
								   	           	   "Number Error",
									           	   JOptionPane.WARNING_MESSAGE);
										return "";						    			
						    		}
						    	}
						    	
						    	if (bodyXML.getNodeName().equals("shoulders")){
								       Node shoulders;
								       Line2D.Float shoulderLine;
								       int x_ls=-1,y_ls=-1,x_rs=-1,y_rs=-1;
									    for(shoulders = bodyXML.getFirstChild();shoulders!= null ;
									    	shoulders = shoulders.getNextSibling()){	
									    	if (shoulders.getNodeName().equals("right")){
									    		try{
										    		String xc_str=((Element) shoulders).getAttribute("xc");
												    String yc_str = ((Element) shoulders).getAttribute("yc");	
												    String state_str = ((Element) shoulders).getAttribute("occluded");	
//												    System.out.println("IN RIGHT state is "+state_str);
												    int xc=Integer.parseInt(xc_str);
												    int yc=Integer.parseInt(yc_str);
												    if (state_str.equals("no"))
												    	body.setVisiblerightshoulder(true);
												    else
												    	body.setVisiblerightshoulder(false);
												    x_rs=xc;y_rs=yc;
									    		}catch(Exception e){
								    				JOptionPane.showMessageDialog(null,
								   				        	  ("Invalid right shoulder dimensions in frame " + fid),
											   	           	   "Number Error",
												           	   JOptionPane.WARNING_MESSAGE);
													return "";										    			
									    		}
									    	}

									    	if (shoulders.getNodeName().equals("left")){
									    		try{
										    		String xc_str=((Element) shoulders).getAttribute("xc");
												    String yc_str = ((Element) shoulders).getAttribute("yc");	
												    String state_str = ((Element) shoulders).getAttribute("occluded");	
												    int xc=Integer.parseInt(xc_str);
												    int yc=Integer.parseInt(yc_str);
//												    System.out.println("IN LEFT state is "+state_str);
												    if (state_str.equals("no"))
												    	body.setVisibleleftshoulder(true);
												    else
												    	body.setVisibleleftshoulder(false);
												    x_ls=xc;y_ls=yc;
										    	}catch(Exception e){
								    				JOptionPane.showMessageDialog(null,
								   				        	  ("Invalid right shoulder dimensions in frame " + fid),
											   	           	   "Number Error",
												           	   JOptionPane.WARNING_MESSAGE);
													return "";										    			
										    	}
									    	}
									    }
									    if ((x_ls!=-1) && (y_ls!=-1) && (x_rs!=-1) && (y_rs!=-1)){
								    		shoulderLine = new Line2D.Float(x_rs,y_rs,x_ls,y_ls);
								    		body.setShoulderLine(shoulderLine);
//								    		System.out.println("shoulder line from right "+x_rs+" "+y_rs+" occluded "+body.isVisiblerightshoulder()
//								    							+" left "+x_ls+" "+y_ls+" occluded "+body.isVisibleleftshoulder());
								    	}
	
						    	}//</shoulders>
						    	
						    	if (bodyXML.getNodeName().equals("hands")){
								       Node hands;
									    for(hands = bodyXML.getFirstChild();hands!= null ;
									    	hands = hands.getNextSibling()){	
									    	if (hands.getNodeName().equals("right")){
									    		try{
										    		String xc_str=((Element) hands).getAttribute("xc");
												    String yc_str = ((Element) hands).getAttribute("yc");	
												    String radious_str = ((Element) hands).getAttribute("size");	
												    String state_str = ((Element) hands).getAttribute("occluded");	
//												    System.out.println("IN RIGHT state for hand is "+state_str);
												    int xc=Integer.parseInt(xc_str);
												    int yc=Integer.parseInt(yc_str);
												    int radious=Integer.parseInt(radious_str);
												    if (state_str.equals("no"))
												    	body.setVisiblerighthand(true);
												    else
												    	body.setVisiblerighthand(false);
												    Circle righthand=new Circle(xc,yc,radious,Circle.RIGHTHAND);
												    body.setRighthand(righthand);
//												    System.out.println("setting right hand to "+xc+" yc "+yc+" radious "+radious+" state "+body.isVisiblerighthand());
									    		}catch(Exception e){
								    				JOptionPane.showMessageDialog(null,
								   				        	  ("Invalid right hand dimensions in frame " + fid),
											   	           	   "Number Error",
												           	   JOptionPane.WARNING_MESSAGE);
													return "";										    			
									    		}
									    	}

									    	if (hands.getNodeName().equals("left")){
									    		try{
										    		String xc_str=((Element) hands).getAttribute("xc");
												    String yc_str = ((Element) hands).getAttribute("yc");	
												    String radious_str = ((Element) hands).getAttribute("size");	
												    String state_str = ((Element) hands).getAttribute("occluded");	
												    int xc=Integer.parseInt(xc_str);
												    int yc=Integer.parseInt(yc_str);
												    int radious=Integer.parseInt(radious_str);
//												    System.out.println("IN LEFT state for hand is "+state_str);
												    if (state_str.equals("no"))
												    	body.setVisiblelefthand(true);
												    else
												    	body.setVisiblelefthand(false);
												    Circle lefthand=new Circle(xc,yc,radious,Circle.LEFTHAND);
												    body.setLefthand(lefthand);
//												    System.out.println("setting left hand to "+xc+" yc "+yc+" radious "+radious+" state "+body.isVisiblelefthand());
									    		}catch(Exception e){
								    				JOptionPane.showMessageDialog(null,
								   				        	  ("Invalid left hand dimensions in frame " + fid),
											   	           	   "Number Error",
												           	   JOptionPane.WARNING_MESSAGE);
													return "";										    			
										    	}
									    	}
									    }
						    	}//</hands>						    	

						    	if (bodyXML.getNodeName().equals("feet")){
								       Node feet;
									    for(feet = bodyXML.getFirstChild();feet!= null ;
									    	feet = feet.getNextSibling()){	
									    	if (feet.getNodeName().equals("right")){
									    		try{
										    		String xc_str=((Element) feet).getAttribute("xc");
												    String yc_str = ((Element) feet).getAttribute("yc");	
												    String radious_str = ((Element) feet).getAttribute("size");	
												    String state_str = ((Element) feet).getAttribute("occluded");	
//												    System.out.println("IN RIGHT state for feet is "+state_str);
												    int xc=Integer.parseInt(xc_str);
												    int yc=Integer.parseInt(yc_str);
												    int radious=Integer.parseInt(radious_str);
												    if (state_str.equals("no"))
												    	body.setVisiblerightfoot(true);
												    else
												    	body.setVisiblerightfoot(false);
												    Circle rightfoot=new Circle(xc,yc,radious,Circle.RIGHTFOOT);
												    body.setRightfoot(rightfoot);
//												    System.out.println("setting right foot to "+xc+" yc "+yc+" radious "+radious+" state "+body.isVisiblerightfoot());
									    		}catch(Exception e){
								    				JOptionPane.showMessageDialog(null,
								   				        	  ("Invalid right foot dimensions in frame " + fid),
											   	           	   "Number Error",
												           	   JOptionPane.WARNING_MESSAGE);
													return "";										    			
									    		}
									    	}

									    	if (feet.getNodeName().equals("left")){
									    		try{
										    		String xc_str=((Element) feet).getAttribute("xc");
												    String yc_str = ((Element) feet).getAttribute("yc");	
												    String radious_str = ((Element) feet).getAttribute("size");	
												    String state_str = ((Element) feet).getAttribute("occluded");	
												    int xc=Integer.parseInt(xc_str);
												    int yc=Integer.parseInt(yc_str);
												    int radious=Integer.parseInt(radious_str);
//												    System.out.println("IN LEFT state for feet is "+state_str);
												    if (state_str.equals("no"))
												    	body.setVisibleleftfoot(true);
												    else
												    	body.setVisibleleftfoot(false);
												    Circle leftfoot=new Circle(xc,yc,radious,Circle.LEFTFOOT);
												    body.setLeftfoot(leftfoot);
//												    System.out.println("setting left foot to "+xc+" yc "+yc+" radious "+radious+" state "+body.isVisibleleftfoot());
									    		}catch(Exception e){
								    				JOptionPane.showMessageDialog(null,
								   				        	  ("Invalid left foot dimensions in frame " + fid),
											   	           	   "Number Error",
												           	   JOptionPane.WARNING_MESSAGE);
													return "";										    			
										    	}
									    	}
									    }
						    	}//</foot>							    	
						    	
						    }
					       
				    }//</body>			    
	
				   //<appearance>
				   if(attribute.getNodeName().equals("appearance")){
				       Node n;
				       for(n = attribute.getFirstChild(); 
					       !n.getNodeName().equals("#text")&& n != null ; 
					       n = n.getNextSibling());
				       
				       String appearance = n.getNodeValue();
				       
				       if(appearance.equals("appear"))
					   sTruths[SingleBox.APPEAR] = true;
				       else if(appearance.equals("disappear"))
					   sTruths[SingleBox.DISAPPEAR] = true;
				       else if(appearance.equals("occluded"))
					   sTruths[SingleBox.OCCLUDED] = true;
				   }//</appearance>
	
				   
				   if(attribute.getNodeName().equals("hypothesislist")){
	//			   	System.out.println("found a hypothesis list");	
				   	Node hypolist;
		    			for(hypolist=attribute.getFirstChild();hypolist!= null; hypolist=hypolist.getNextSibling()){				   
					    if (hypolist.getNodeName().equals("hypothesis")){
	//				    		System.out.println("find the hypothesis");
							Node hypo;
				    			for(hypo=hypolist.getFirstChild();hypo!= null; hypo=hypo.getNextSibling()){					    
	
								   //<movement>
								   if(hypo.getNodeName().equals("movement")){
	//								System.out.println("found movement");							   
								       Node n;
								       for(n = hypo.getFirstChild();!n.getNodeName().equals("#text")&& n != null;  
									       n = n.getNextSibling());
								       
								       String movement = n.getNodeValue();
	
								       double eval = getEval(hypo, fid);
								       if(eval < 0) return "";
					
								       if(movement.equals("inactive")){
									   sTruths[SingleBox.INACTIVE] = true;
									   sTruths[SingleBox.NONE_MOV] = false;
								       }
								       else if(movement.equals("active")){
									   sTruths[SingleBox.ACTIVE] = true;
									   sTruths[SingleBox.NONE_MOV] = false;
								       }
								       else if(movement.equals("walking")){
									   sTruths[SingleBox.WALK] = true;
									   sTruths[SingleBox.NONE_MOV] = false;
								       }
								       else if(movement.equals("running")){
									   sTruths[SingleBox.RUN] = true;
									   sTruths[SingleBox.NONE_MOV] = false;
								       }
								   }//</movement>
	
								   //<role>
								   if(hypo.getNodeName().equals("role")){
	//								System.out.println("found role");	
								       Node n;
								       for(n = hypo.getFirstChild();!n.getNodeName().equals("#text")&& n != null; 
									       n = n.getNextSibling()); 
								       
								       String role = n.getNodeValue();
					
								       double eval = getEval(hypo, fid);
								       if(eval < 0) return "";
					
								       if(role.equals("fighter")){
									   sTruths[SingleBox.FIGHTER] = true;
									   sTruths[SingleBox.NONE_ROLE] = false;
								       }
								       else if(role.equals("browser")){
									   sTruths[SingleBox.BROWSER] = true;
									   sTruths[SingleBox.NONE_ROLE] = false;
								       }
								       else if(role.equals("leaving victim")){
									   sTruths[SingleBox.LEFT_VICTIM] = true;
									   sTruths[SingleBox.NONE_ROLE] = false;
								       }
								       else if(role.equals("leaving group")){
	/*								   if(!lgwarned){
									       lgwarned = true;
						    				JOptionPane.showMessageDialog(null,
					   				        	  ("This file contains \"leaving group\" role"),
								   	           	   "Compatibility Error",
									           	   JOptionPane.WARNING_MESSAGE);
									   }*/
									   sTruths[SingleBox.LEAVING_GROUP] = true;
									   sTruths[SingleBox.NONE_ROLE] = false;
								       }
								       else if(role.equals("walker")){
									   sTruths[SingleBox.WALKER] = true;
									   sTruths[SingleBox.NONE_ROLE] = false;
								       }
								       else if(role.equals("leaving object")){
									   sTruths[SingleBox.LEFT_OBJECT] = true;
									   sTruths[SingleBox.NONE_ROLE] = false;
								       }
					
								   }//</role>
	
								   //<scenario>
								   if(hypo.getNodeName().equals("context")){
	//								System.out.println("found context");	
								       Node n;
								       for(n = hypo.getFirstChild();
									       !n.getNodeName().equals("#text")&& n != null; 
									       n = n.getNextSibling());
								       
								       String scenario = n.getNodeValue();
					
								       double eval = getEval(hypo, fid);
								       if(eval < 0) return "";
					
								       if(scenario.equals("browsing")){
									   sTruths[SingleBox.BROWSING] = true;
									   sTruths[SingleBox.NONE_SCE] = false;
								       }
								       else if(scenario.equals("immobile")){
									   sTruths[SingleBox.IMMOBILE] = true;
									   sTruths[SingleBox.NONE_SCE] = false;
								       }
								       else if(scenario.equals("leaving object")){
									   sTruths[SingleBox.LEFT_OBJECT] = true;
									   sTruths[SingleBox.NONE_SCE] = false;
								       }
								       else if(scenario.equals("walking")){
									   sTruths[SingleBox.WALKING] = true;
									   sTruths[SingleBox.NONE_SCE] = false;
								       }
								       else if(scenario.equals("drop down")){
									   sTruths[SingleBox.DROP_DOWN] = true;
									   sTruths[SingleBox.NONE_SCE] = false;
								       }
								       else if(scenario.equals("windowshop")){
									   sTruths[SingleBox.WINDOWSHOP_SCE] = true;
									   sTruths[SingleBox.NONE_SCE] = false;
								       }
								       else if(scenario.equals("shop enter")){
									   sTruths[SingleBox.SHOPENTER_SCE] = true;
									   sTruths[SingleBox.NONE_SCE] = false;
								       }
								       else if(scenario.equals("shop reenter")){
									   sTruths[SingleBox.SHOPREENTER_SCE] = true;
									   sTruths[SingleBox.NONE_SCE] = false;
								       }
								       else if(scenario.equals("shop exit")){
									   sTruths[SingleBox.SHOPEXIT_SCE] = true;
									   sTruths[SingleBox.NONE_SCE] = false;
								       }
								       
								   }//</scenario>
								
	
								   //<situation>
								   if(hypo.getNodeName().equals("situation")){
	//								System.out.println("found situation");
								       Node n;
								       for(n = hypo.getFirstChild();
									       !n.getNodeName().equals("#text")&& n != null; 
									       n = n.getNextSibling());
								       
								       String situation = n.getNodeValue();
					
								       double eval = getEval(hypo, fid);
								       if(eval < 0) return "";
					
								       if(situation.equals("moving")){
									   sTruths[SingleBox.MOVING_SIT] = true;
									   sTruths[SingleBox.NONE_SIT] = false;
								       }
								       else if(situation.equals("inactive")){
									   sTruths[SingleBox.INACTIVE_SIT] = true;
									   sTruths[SingleBox.NONE_SIT] = false;
								       }
								       else if(situation.equals("browsing")){
									   sTruths[SingleBox.BROWSING_SIT] = true;
									   sTruths[SingleBox.NONE_SIT] = false;
								       }
								       else if(situation.equals("shop enter")){
									   sTruths[SingleBox.SHOPENTER_SIT] = true;
									   sTruths[SingleBox.NONE_SIT] = false;
								       }
								       else if(situation.equals("shop exit")){
									   sTruths[SingleBox.SHOPEXIT_SIT] = true;
									   sTruths[SingleBox.NONE_SIT] = false;
								       }
					
								   }//</situation>
											
							}
					    }  
	  				}
				   }
				   
				   //<event>
				   if(attribute.getNodeName().equals("event")){
				       Node n;
				       for(n = attribute.getFirstChild();
					       !n.getNodeName().equals("#text")&& n != null; 
					       n = n.getNextSibling());
				       
				       String event = n.getNodeValue();
	
				       double eval = getEval(attribute, fid);
				       if(eval < 0) return "";
	
				       if(event.equals("immobile")){
					   sTruths[SingleBox.PERSON_IMMOBILE] = true;
				       }
				   }//</event>
	
				}//</orientation>, </box>, </appearance>, </movement>, </role>
				//</scenario>, </situation>, </event>
	
				if(!foundOrientation){
		    		    JOptionPane.showMessageDialog(null,
	   	        	  	("No orientation in frame " + fid),
		             	   	"Orientation Error",
		           	   	JOptionPane.WARNING_MESSAGE);
				   	return "";
				}
	
				if(!foundBox){
		    		    JOptionPane.showMessageDialog(null,
	   	        	  	("No box in frame " + fid),
		             	   	"Box Error",
		           	   	JOptionPane.WARNING_MESSAGE);
				   	return "";
				}
			
				Point[] linePoints = getAxisPoints(axis, (int) box.getX(), (int) 
					box.getY(), (int) box.getWidth(), (int) box.getHeight());
				line = new Line2D.Float((float)linePoints[0].x,(float)linePoints[0].y,
							    (float)linePoints[1].x,(float)linePoints[1].y);
				String idString = ((Element) element).getAttribute("id");
				if(idString.equals("")){
		    		    JOptionPane.showMessageDialog(null,
	   	        	  	("Invalid id " + idString + " in frame " + fid),
		             	   	"Number Error",
		           	   	JOptionPane.WARNING_MESSAGE);
				   	return "";
				}
				try{
				    int id = Integer.parseInt(idString);
				    sBoxes[id] = new SingleBox(sTruths, box, axis, line , body);
				}
				catch(NumberFormatException e){
		    		    JOptionPane.showMessageDialog(null,
	   	        	  	("Invalid id " + idString + " in frame " + fid),
		             	   	"Number Error",
		           	   	JOptionPane.WARNING_MESSAGE);
				   	return "";
				}
					
			    }//</object>
			}//</objectlist>
	
			//<grouplist>
			if(list.getNodeName().equals("grouplist")){
			
			    //<group>
			    for(element = list.getFirstChild(); 
				    element != null; element = element.getNextSibling()){
	
				if(!element.getNodeName().equals("group")) continue;
				
				// reset all attributes
				for (int j = 0; j < gTruths.length; j++) {
				    gTruths[j] = false;
				}
				gTruths[GroupBox.NONE_ROLE] = gTruths[GroupBox.NONE_SIT] =
				    gTruths[GroupBox.NONE_SCE] = gTruths[GroupBox.NONE_MOV] = true;
				
				box = null;
				axis = 90;
				for(int j = 0; j<gIndBoxes.length; j++){
				    gIndBoxes[j] = false;
				}
	
				//<orientation>, <box>, <appearance>, <movement>, <role>
				//<scenario>, <situation>, <event> <members>
				for(attribute = element.getFirstChild(); 
					attribute != null; attribute = attribute.getNextSibling()){
				    
				    //<orientation>
				    if(attribute.getNodeName().equals("orientation")){
				       Node n;
				       for(n = attribute.getFirstChild();
					       !n.getNodeName().equals("#text")&& n != null; 
					       n = n.getNextSibling());
					
				       String axisString = n.getNodeValue();	       
				       try{
					    axis = Integer.parseInt(axisString);
					}catch(NumberFormatException e){ 
		    				JOptionPane.showMessageDialog(null,
	   				        	  (axisString + 
							   " not a valid orientation in frame "
							   + fid),
				   	           	   "Number Error",
					           	   JOptionPane.WARNING_MESSAGE);
						return "";
				        }
				    }//</orientation>
	
				    //<box>
				    if(attribute.getNodeName().equals("box")){
					try{
					    String w = ((Element) attribute).getAttribute("w");
					    String h = ((Element) attribute).getAttribute("h");
					    String x = ((Element) attribute).getAttribute("x");
					    String y = ((Element) attribute).getAttribute("y");
					    String xc= ((Element) attribute).getAttribute("xc");
					    String yc= ((Element) attribute).getAttribute("yc");
					    boolean nocenter=((x.equals("") || y.equals("")) && 
					                      (xc.equals("") || yc.equals("")));
					    boolean center=(xc.equals("") || yc.equals(""));
					    if(w.equals("") || h.equals("") || nocenter){
						JOptionPane.showMessageDialog(null,
	   				        	  ("Invalid box dimensions in frame " + fid),
				   	           	   "Number Error",
					           	   JOptionPane.WARNING_MESSAGE);
						return "";
					    }
					    int wi = Integer.parseInt(w);
					    int hi = Integer.parseInt(h);
					    int xi=0,yi=0;
					    if (center==true){
						    xi = Integer.parseInt(x);
						    yi = Integer.parseInt(y);
						 }else{
						 	 xi= Integer.parseInt(xc)- wi/2;
							 yi= Integer.parseInt(yc)- hi/2;
						 }
					    box = new Rectangle(xi,yi,wi,hi);
					}
					catch(NumberFormatException e){ 
					    JOptionPane.showMessageDialog(null,
	   				          ("Invalid box dimensions in frame " + fid),
				   	           "Number Error",
					      	   JOptionPane.WARNING_MESSAGE);
					    return "";
					}
				    }//</box>
	
				    //<members>
				    if(attribute.getNodeName().equals("members")){
				        Node n;
				        for(n = attribute.getFirstChild();
					       !n.getNodeName().equals("#text")&& n != null; 
					       n = n.getNextSibling());
					    
				        String[] membs = n.getNodeValue().split(",");
					for(int j=0; j<membs.length; j++){
					    membs[j] = membs[j].trim();
					    try{
						gIndBoxes[Integer.parseInt(membs[j])] = true;
	
					    }catch(NumberFormatException e){
		    				JOptionPane.showMessageDialog(null,
	   				        	  ("Invalid group members in frame " + fid),
				   	           	   "Number Error",
					           	   JOptionPane.WARNING_MESSAGE);
						return "";
					    }
					}
	
				    }//</members>
	
				   //<appearance>
				   if(attribute.getNodeName().equals("appearance")){
				       Node n;
				       for(n = attribute.getFirstChild();
					       !n.getNodeName().equals("#text")&& n != null; 
					       n = n.getNextSibling());
				       
				       String appearance = n.getNodeValue();
				   
				       if(appearance.equals("appear"))
					   gTruths[GroupBox.APPEAR] = true;
				       else if(appearance.equals("disappear"))
					   gTruths[GroupBox.DISAPPEAR] = true;
				   
				   }//</appearance>
	
				   if(attribute.getNodeName().equals("hypothesislist")){
	//			   	System.out.println("found a hypothesis list for group");	
				   	Node hypolist;
		    			for(hypolist=attribute.getFirstChild();hypolist!= null; hypolist=hypolist.getNextSibling()){				   
					    if (hypolist.getNodeName().equals("hypothesis")){
	//				    		System.out.println("find the hypothesis for group");
							Node hypo;
				    			for(hypo=hypolist.getFirstChild();hypo!= null; hypo=hypo.getNextSibling()){					    
							   //<movement>
							   if(hypo.getNodeName().equals("movement")){
	//						   	  System.out.println("movement in group");
							       Node n;
							       for(n = hypo.getFirstChild();
								       !n.getNodeName().equals("#text")&& n != null; 
								       n = n.getNextSibling());
							       
							       String movement = n.getNodeValue();
	
							       double eval = getEval(hypo, fid);
							       if(eval < 0) return "";
							   
							       if(movement.equals("inactive")){
	//						        System.out.println("readed inactive");						       
								   gTruths[GroupBox.INACTIVE] = true;
								   gTruths[GroupBox.NONE_MOV] = false;
							       }
							       else if(movement.equals("active")){
	//						        System.out.println("readed active");
								   gTruths[GroupBox.ACTIVE] = true;
								   gTruths[GroupBox.NONE_MOV] = false;
							       }
							       else if(movement.equals("movement")){
	//						        System.out.println("readed movement");
								   gTruths[GroupBox.MOVE] = true;
								   gTruths[GroupBox.NONE_MOV] = false;
							       }
				
							   }//</movement>
	
							   //<role>
							   if(hypo.getNodeName().equals("role")){
	//						   	  System.out.println("role in group");						   
							       Node n;
							       for(n = hypo.getFirstChild();
								       !n.getNodeName().equals("#text")&& n != null; 
								       n = n.getNextSibling());
							       String role = n.getNodeValue();
				
							       double eval = getEval(hypo, fid);
							       if(eval < 0) return "";
				
							       if(role.equals("fighters")){
								   gTruths[GroupBox.FIGHTERS] = true;
								   gTruths[GroupBox.NONE_ROLE] = false;
							       }
							       if(role.equals("meeters")){
								   gTruths[GroupBox.MEETERS] = true;
								   gTruths[GroupBox.NONE_ROLE] = false;
							       }
							       else if(role.equals("browsers")){
								   gTruths[GroupBox.GRBROWSER_ROLE] = true;
								   gTruths[GroupBox.NONE_ROLE] = false;
							       }
							       else if(role.equals("leaving victim")){
								   gTruths[GroupBox.GRLEFTVICTIM_ROLE] = true;
								   gTruths[GroupBox.NONE_ROLE] = false;
							       }
							       else if(role.equals("walkers")){
								   gTruths[GroupBox.WALKERS] = true;
								   gTruths[GroupBox.NONE_ROLE] = false;
							       }
							       else if(role.equals("leaving object")){
								   gTruths[GroupBox.GRLEFTOBJECT_ROLE] = true;
								   gTruths[GroupBox.NONE_ROLE] = false;
							       }
							   }//</role>
	
							   //<context>
							   if(hypo.getNodeName().equals("context")){
	//						   	  System.out.println("context in group");							   
							       Node n;
							       for(n = hypo.getFirstChild();
								       !n.getNodeName().equals("#text")&& n != null; 
								       n = n.getNextSibling());
							       
							       String scenario = n.getNodeValue();
				
							       double eval = getEval(hypo, fid);
							       if(eval < 0) return "";
							       
	//						       System.out.println("scenario is "+scenario);
				
							       if(scenario.equals("fighting")){
								   gTruths[GroupBox.FIGHTING] = true;
								   gTruths[GroupBox.NONE_SCE] = false;
							       }
							       if(scenario.equals("meeting")){
								   gTruths[GroupBox.MEETING] = true;
								   gTruths[GroupBox.NONE_SCE] = false;
							       }
							       if(scenario.equals("leaving")){
								   gTruths[GroupBox.LEAVING] = true;
								   gTruths[GroupBox.NONE_SCE] = false;
							       }
							       if(scenario.equals("walking")){
								   gTruths[GroupBox.GRWALKING_SCE] = true;
								   gTruths[GroupBox.NONE_SCE] = false;
							       }
							       if(scenario.equals("windowshop")){
								   gTruths[GroupBox.GRWINDOWSHOP_SCE] = true;
								   gTruths[GroupBox.NONE_SCE] = false;
							       }
							       if(scenario.equals("shop enter")){
								   gTruths[GroupBox.GRSHOPENTER_SCE] = true;
								   gTruths[GroupBox.NONE_SCE] = false;
							       }
							       if(scenario.equals("shop reenter")){
								   gTruths[GroupBox.GRSHOPREENTER_SCE] = true;
								   gTruths[GroupBox.NONE_SCE] = false;
							       }
							       if(scenario.equals("shop exit")){
								   gTruths[GroupBox.GRSHOPEXIT_SCE] = true;
								   gTruths[GroupBox.NONE_SCE] = false;
							       }
							   }//</scenario>
							   
							   //<situation>
							   if(hypo.getNodeName().equals("situation")){
	  // 						   	  System.out.println("situation in group");
							       Node n;
							       for(n = hypo.getFirstChild();
								       !n.getNodeName().equals("#text")&& n != null; 
								       n = n.getNextSibling());
							       String situation = n.getNodeValue();
							   
							       double eval = getEval(hypo, fid);
							       if(eval < 0) return "";
							       
	//						       System.out.println("situation is "+situation);
				
							       if(situation.equals("split up")){
								   gTruths[GroupBox.GR_SPLITTING] = true;
								   gTruths[GroupBox.NONE_SIT] = false;
							       }
							       if(situation.equals("fighting")){
								   gTruths[GroupBox.GR_FIGHTING] = true;
								   gTruths[GroupBox.NONE_SIT] = false;
							       }
							       if(situation.equals("moving")){
								   gTruths[GroupBox.GR_MOVING] = true;
								   gTruths[GroupBox.NONE_SIT] = false;
							       }
							       if(situation.equals("joining")){
								   gTruths[GroupBox.GR_MEETING] = true;
								   gTruths[GroupBox.NONE_SIT] = false;
							       }
							       if(situation.equals("interacting")){
								   gTruths[GroupBox.GR_INTERACTING] = true;
								   gTruths[GroupBox.NONE_SIT] = false;
							       }
							       if(situation.equals("inactive")){
								   gTruths[GroupBox.GR_INACTIVE] = true;
								   gTruths[GroupBox.NONE_SIT] = false;
							       }
							       if(situation.equals("leaving victim")){
								   gTruths[GroupBox.GR_LEFT_VICTIM] = true;
								   gTruths[GroupBox.NONE_SIT] = false;
							       }
							       if(situation.equals("leaving object")){
								   gTruths[GroupBox.GR_LEFT_OBJECT] = true;
								   gTruths[GroupBox.NONE_SIT] = false;
							       }
							       if(situation.equals("shop enter")){
								   gTruths[GroupBox.GRSHOPENTER_SIT] = true;
								   gTruths[GroupBox.NONE_SIT] = false;
							       }
							       if(situation.equals("shop exit")){
								   gTruths[GroupBox.GRSHOPEXIT_SIT] = true;
								   gTruths[GroupBox.NONE_SIT] = false;
							       }
							       if(situation.equals("browsing")){
								   gTruths[GroupBox.GRBROWSING_SIT] = true;
								   gTruths[GroupBox.NONE_SIT] = false;
							       }
							   }//</situation>
	
							}
						}//</hypothesis>
					}
				   }//</hypothesislist>
	
	
	
				       
	
	
	
	
	
				   
				   //<event>
				   if(attribute.getNodeName().equals("event")){
				       Node n;
				       for(n = attribute.getFirstChild();
					       !n.getNodeName().equals("#text")&& n != null; 
					       n = n.getNextSibling());
				       
			      String event = n.getNodeValue();
	
				       double eval = getEval(attribute, fid);
				       if(eval < 0) return "";
	
				       if(event.equals("fight start")){
					   gTruths[GroupBox.FIGHT_START] = true;
				       }
				       if(event.equals("fight end")){
					   gTruths[GroupBox.FIGHT_END] = true;
				       }
				       if(event.equals("left victim")){
					   gTruths[GroupBox.FIGHT_VICTIM_LEFT] = true;
				       }
				       
				   }//</event>
				}//</orientation>, </box>, </appearance>, </movement>, </role>
				//</scenario>, </situation>, </event> </memebers>
			    
				Point[] linePoints = getAxisPoints(axis, (int) box.getX(), (int) 
					box.getY(), (int) box.getWidth(), (int) box.getHeight());
				line = new Line2D.Float((float)linePoints[0].x,(float)linePoints[0].y,
							    (float)linePoints[1].x,(float)linePoints[1].y);
				String idString = ((Element) element).getAttribute("id");
				
				if(idString.equals("")){
		    		    JOptionPane.showMessageDialog(null,
	   	        	  	("Invalid id " + idString + " in frame " + fid),
		             	   	"Number Error",
		           	   	JOptionPane.WARNING_MESSAGE);
				   	return "";
				}
				try{
				    int id = Integer.parseInt(idString);
				    gBoxes[id] = new GroupBox(gTruths, box, axis, line, gIndBoxes);
			        }
				catch(NumberFormatException e){
		    		    JOptionPane.showMessageDialog(null,
	   	        	  	("Invalid id " + idString + " in frame " + fid),
		             	   	"Number Error",
		           	   	JOptionPane.WARNING_MESSAGE);
				   	return "";
				}
	
					
			    }//</group>
			    
			}//</grouplist>
			
		    }//<objectlist> , <grouplist>
	
		    frameData[fid] = new FrameData(sBoxes, gBoxes);
		    highest = fid > highest ? fid : highest;
		}//</frame>
		}catch (Exception e){
			System.out.println("Error loading frame number "+(framenumber-1));
			e.printStackTrace();
		}
		seqName = ((Element) doc.getDocumentElement()).getAttribute("name");
		return seqName+"\n"+highest;
	    }

    /*
     * Auxiliary method used by readFrom to create the required
     * objects to represent the information gathered in readString
	* HUGO:
	* I couldn't cast the whole Object[] array into String[], so 
	* casts were done independently for each array element, i.e. (String)data[x]
	* Although it now works, It'll be better to do it in one step!... any volunteer?
	* Method now obsolete - public String updateData(Document doc) used instead
	* PTC Method now obsolete
	*/
    /*public String updateData(Object [] data) {

	SingleBox[] sBoxes =  new SingleBox[FrameData.SINGLE_BOXES];

	for (int i = 0; i < sBoxes.length; i++) {
	    sBoxes[i] = new SingleBox();
	}

	boolean[] sTruths = new boolean[SingleBox.LENGTH];
	
	GroupBox[] gBoxes = new GroupBox[FrameData.GROUP_BOXES];

	for (int i = 0; i < gBoxes.length; i++) {
	    gBoxes[i] = new GroupBox();
	}

	boolean[] gTruths = new boolean[GroupBox.LENGTH];

	boolean[] gIndBoxes = new boolean[FrameData.SINGLE_BOXES];

	int axis = 90;

	Rectangle box = new Rectangle();

	Line2D.Float line = new Line2D.Float();

	String seqName = "";
	
	// frame number in sequence
	int frameId = 0;

	// box number (in array)
	int boxNo = 0;

	// flag to indicate furthest picture found so far
	int highest = 0;
	int limit   = 0;

	frameData = new FrameData[FRAMES];
	System.out.println("Total variables = " + data.length + "\nProgress:");
	for (int i = 0; i < data.length; i++) {
		if (i%1000==0)
		    System.out.print(i*100/data.length + "% ");

	    if ( ((String)data[i]).equals("frame")) {
		// case of beginning of new frame
		continue;
	    } else if (i > 0 && ((String)data[i-1]).equals("frame")) {
		// case of found sequence name
		seqName = (String)data[i];
	    } else if (i > 1 && ((String)data[i-2]).equals("frame")) {
		// case of found frame id (number of frame in sequence)
		frameId = Integer.parseInt((String)data[i]);
	    } else if ( ((String)data[i]).equals("ibl")) {
		// case of single box list
		++i;
	        while(!((String)data[i]).equals("eibl")) {

		    if ( ((String)data[i]).equals("eib")) {
			// create representative single box
			sBoxes[boxNo] = new SingleBox(sTruths,box,axis,line);
			++i;
		    } else if ( ((String)data[i]).equals("ib")) {
			// start of a single box
			++i;
			boxNo = Integer.parseInt((String)data[i]);
			++i;
			// reset all attributes
			for (int j = 0; j < sTruths.length; j++) {
			    sTruths[j] = false;
			}
			box = null;
			axis = 90;
			
			if (!((String)data[i]).equals("eib")) {

			    int width = Integer.parseInt((String)data[i+2]);
			    int height = Integer.parseInt((String)data[i+3]);
			    // now specifying central coords
			    int x = Integer.parseInt((String)data[i])-(width/2);
			    int y = Integer.parseInt((String)data[i+1])-(height/2);
			    box = new Rectangle(x, y, width, height);
			    axis = Integer.parseInt((String)data[i+4]);
			    Point[] linePoints = getAxisPoints(axis,x,y,width,height);
			    line = new Line2D.Float((float)linePoints[0].x,(float)linePoints[0].y,
						    (float)linePoints[1].x,(float)linePoints[1].y);
			    i = i + 5;
			    
			    while (!((String)data[i]).equals("eib")) {
				// APPEARANCE
				if ( ((String)data[i]).equals("ap")) {
				    sTruths[SingleBox.APPEAR] = true;
				} else if ( ((String)data[i]).equals("di")) {
				    sTruths[SingleBox.DISAPPEAR] = true;
				} else if ( ((String)data[i]).equals("o")) {
				    sTruths[SingleBox.OCCLUDED] = true;
				}
				// MOVEMENTS
				else if ( ((String)data[i]).equals("wk")) {
				    sTruths[SingleBox.WALK] = true;
				} else if ( ((String)data[i]).equals("r")) {
				    sTruths[SingleBox.RUN] = true;
				} else if ( ((String)data[i]).equals("ac")) {
				    sTruths[SingleBox.ACTIVE] = true;
				} else if ( ((String)data[i]).equals("in")) {
				    sTruths[SingleBox.INACTIVE] = true;
				} 
				// ROLES
				else if ( ((String)data[i]).equals("f")) {
				   sTruths[SingleBox.FIGHTER] = true;
				} else if ( ((String)data[i]).equals("br")) {
				   sTruths[SingleBox.BROWSER] = true;
				} else if ( ((String)data[i]).equals("lv")) {
				    sTruths[SingleBox.LEFT_VICTIM] = true;
				} else if ( ((String)data[i]).equals("lg")) {
				    sTruths[SingleBox.LEAVING_GROUP] = true;
				} else if ( ((String)data[i]).equals("wr")) {
				    sTruths[SingleBox.WALKER] = true;
				} else if ( ((String)data[i]).equals("lo")) {
				    sTruths[SingleBox.LEFT_OBJECT] = true;
				}
				// SITUATIONS
				else if ( ((String)data[i]).equals("m")) {
				    sTruths[SingleBox.MOVING_SIT] = true;
				} else if ( ((String)data[i]).equals("is")) {
				    sTruths[SingleBox.INACTIVE_SIT] = true;
				} else if ( ((String)data[i]).equals("bsi")) {
				    sTruths[SingleBox.BROWSING_SIT] = true;
				} else if ( ((String)data[i]).equals("iext")) {
				    sTruths[SingleBox.SHOPEXIT_SIT] = true;
				} else if ( ((String)data[i]).equals("ient")) {
				    sTruths[SingleBox.SHOPENTER_SIT] = true;
				}
				// SCENARIOS
				else if ( ((String)data[i]).equals("bsc")) {
				    sTruths[SingleBox.BROWSING] = true;
				} else if ( ((String)data[i]).equals("im")) {
				    sTruths[SingleBox.IMMOBILE] = true;
				} else if ( ((String)data[i]).equals("wg")) {
				    sTruths[SingleBox.WALKING] = true;
				} else if ( ((String)data[i]).equals("dd")) {
				    sTruths[SingleBox.DROP_DOWN] = true;
				} else if ( ((String)data[i]).equals("iwsp")) {
				    sTruths[SingleBox.WINDOWSHOP_SCE] = true;
				} else if ( ((String)data[i]).equals("isnt")) {
				    sTruths[SingleBox.SHOPENTER_SCE] = true;
				} else if ( ((String)data[i]).equals("isxt")) {
				    sTruths[SingleBox.SHOPEXIT_SCE] = true;
				} else if ( ((String)data[i]).equals("iren")) {
				    sTruths[SingleBox.SHOPREENTER_SCE] = true;
				}
				// EVENTS
				else if ( ((String)data[i]).equals("pi")) {
				    sTruths[SingleBox.PERSON_IMMOBILE] = true;
				}

				++i;
			    } 
			}
		    }
		} 
	    }
	    
	    else if ( ((String)data[i]).equals("gbl")) {
		// case of group box list
		++i;
		while(!((String)data[i]).equals("egbl")) {
		    if ( ((String)data[i]).equals("egb")) {
			// create representative single box
			gBoxes[boxNo] = new GroupBox(gTruths,box,axis,line,gIndBoxes);
			i++;
		    
		    } else if ( ((String)data[i]).equals("gb")) {
			// start of a group box
			++i;
			boxNo = Integer.parseInt((String)data[i]);;
			++i;
			// reset all attributes
			for (int j = 0; j < gTruths.length; j++) {
			    gTruths[j] = false;
			}
			for (int j = 0; j < gIndBoxes.length; j++) {
			    gIndBoxes[j] = false;
			}
			box = null;
			axis = 90;

			if (!((String)data[i]).equals("egb")) {
			    int width = Integer.parseInt((String)data[i+2]);
			    int height = Integer.parseInt((String)data[i+3]);
			    // new coords means that no longer specify the top left corner
			    int x = Integer.parseInt((String)data[i])-(width/2);
			    int y = Integer.parseInt((String)data[i+1])-(height/2);
			    box = new Rectangle(x, y, width, height);
			    axis = Integer.parseInt((String)data[i+4]);
			    Point[] linePoints = getAxisPoints(axis,x,y,width,height);
			    line = new Line2D.Float((float)linePoints[0].x,(float)linePoints[0].y,
						    (float)linePoints[1].x,(float)linePoints[1].y);
			    i = i + 5;

			    if ( ((String)data[i]).equals("gibl")) {
				++i;
				while (!((String)data[i]).equals("egibl")) {
				    try {
					gIndBoxes[Integer.parseInt((String)data[i])] = true;
				    } catch (NumberFormatException nfe) {
					nfe.printStackTrace();
				    }
				    ++i;
				}
			    }

			    while (!((String)data[i]).equals("egb")) {
				// APPEARANCE
				if ( ((String)data[i]).equals("ap")) {
				    gTruths[GroupBox.APPEAR] = true;
				} else if ( ((String)data[i]).equals("d")) {
				    gTruths[GroupBox.DISAPPEAR] = true;
				} 
				// MOVEMENT
				else if ( ((String)data[i]).equals("ac")) {
				    gTruths[GroupBox.ACTIVE] = true;
				} else if ( ((String)data[i]).equals("i")) {
				    gTruths[GroupBox.INACTIVE] = true;
				} else if ( ((String)data[i]).equals("mo")) {
				    gTruths[GroupBox.MOVE] = true;
				} 
				// ROLES
				else if ( ((String)data[i]).equals("gbr")) {
				    gTruths[GroupBox.GRBROWSER_ROLE] = true;
				} else if ( ((String)data[i]).equals("f")) {
				    gTruths[GroupBox.FIGHTERS] = true;
				} else if ( ((String)data[i]).equals("me")) {
				    gTruths[GroupBox.MEETERS] = true;
				} else if ( ((String)data[i]).equals("w")) {
				    gTruths[GroupBox.WALKERS] = true;
				} 
				// SITUATIONS
				else if ( ((String)data[i]).equals("gf")) {
				    gTruths[GroupBox.GR_FIGHTING] = true;
				} else if ( ((String)data[i]).equals("gmo")) {
				    gTruths[GroupBox.GR_MOVING] = true;
				} else if ( ((String)data[i]).equals("gme")) {
				    gTruths[GroupBox.GR_MEETING] = true;
				} else if ( ((String)data[i]).equals("gint")) {
				    gTruths[GroupBox.GR_INTERACTING] = true;
				} else if ( ((String)data[i]).equals("s")) {
				    gTruths[GroupBox.GR_SPLITTING] = true;
				} else if ( ((String)data[i]).equals("gi")) {
				    gTruths[GroupBox.GR_INACTIVE] = true;
				} else if ( ((String)data[i]).equals("glv")) {
				    gTruths[GroupBox.GR_LEFT_VICTIM] = true;
				} else if ( ((String)data[i]).equals("glo")) {
				    gTruths[GroupBox.GR_LEFT_OBJECT] = true;
				} else if ( ((String)data[i]).equals("gent")) {
				    gTruths[GroupBox.GRSHOPENTER_SIT] = true;
				} else if ( ((String)data[i]).equals("gext")) {
				    gTruths[GroupBox.GRSHOPEXIT_SIT] = true;
				}else if ( ((String)data[i]).equals("gbst")) {
				    gTruths[GroupBox.GRBROWSING_SIT] = true;
				}
				// SCENARIOS
				else if ( ((String)data[i]).equals("fsc")) {
				    gTruths[GroupBox.FIGHTING] = true;
				} else if ( ((String)data[i]).equals("mes")) {
				    gTruths[GroupBox.MEETING] = true;
				} else if ( ((String)data[i]).equals("ls")) {
				    gTruths[GroupBox.LEAVING] = true;
				} else if ( ((String)data[i]).equals("gwsp")) {
				    gTruths[GroupBox.GRWINDOWSHOP_SCE] = true;
				} else if ( ((String)data[i]).equals("gsnt")) {
				    gTruths[GroupBox.GRSHOPENTER_SCE] = true;
				} else if ( ((String)data[i]).equals("gsxt")) {
				    gTruths[GroupBox.GRSHOPEXIT_SCE] = true;
				} else if ( ((String)data[i]).equals("gren")) {
				    gTruths[GroupBox.GRSHOPREENTER_SCE] = true;
				} else if ( ((String)data[i]).equals("gwk")) {
				    gTruths[GroupBox.GRWALKING_SCE] = true;
				}
				// EVENTS
				else if ( ((String)data[i]).equals("fst")) {
				    gTruths[GroupBox.FIGHT_START] = true;
				} else if ( ((String)data[i]).equals("fe")) {
				    gTruths[GroupBox.FIGHT_END] = true;
				} else if ( ((String)data[i]).equals("fv")) {
				    gTruths[GroupBox.FIGHT_VICTIM_LEFT] = true;
				}
				++i;
			    } 
			}
		    }
		}
	    } 

	    else if ( ((String)data[i]).equals("eframe")) {
		frameData[frameId] = new FrameData(sBoxes,gBoxes);
		//System.out.println(frameData[frameId].toString());
		sBoxes =  new SingleBox[FrameData.SINGLE_BOXES];
		for (int k = 0; k < sBoxes.length; k++) {
		    sBoxes[k] = new SingleBox();
		}
		gBoxes = new GroupBox[FrameData.GROUP_BOXES];
		for (int k = 0; k < gBoxes.length; k++) {
		    gBoxes[k] = new GroupBox();
		}
		highest = frameId;
		++i;
	    }
	}
	// returns a string consisting of the sequence name and
	// the number of the furthest frame into the sequence
	// seperated by a dot.
	System.out.print("100%");
	return seqName+"\n"+highest;
    }*/

    /**
     * Similar to setAxisPoints in DrawCanvas, takes an angle and the coordinates 
     * of the top left corner of the rectangle along with the rectangle's height and
     * width to calculate what the line should be for the relavent box object
     */
    public Point[] getAxisPoints(int angle, int x, int y, int width, int height) {
	int x1, y1, x2, y2;
	int centerX = x+width/2;
	int centerY = y+height/2;
	Point linePoint1 = null;
	Point linePoint2 = null;
		
	double rad_angle = (180-angle)* Math.PI / 180;
	    
	// translate to center
	// choose translation so fits in box,
	// ie smallest of height or width
	if (width <= height) {
	    x1 = 0;
	    y1 = 0;
	    x2 = width;
	    y2 = 0;
	} else {
	    x1 = 0;
	    y1 = 0;
	    x2 = height;
		y2 = 0;
	}
	
	// calculate rotation
	int tempX1 = (int) ((Math.cos(rad_angle)*x1) + (-Math.sin(rad_angle)*y1));
	int tempY1 = (int) ((Math.sin(rad_angle)*x1) + (Math.cos(rad_angle)*y1));
	int tempX2 = (int) ((Math.cos(rad_angle)*x2) + (-Math.sin(rad_angle)*y2));
	int tempY2 = (int) ((Math.sin(rad_angle)*x2) + (Math.cos(rad_angle)*y2));
	
	// translate to point where we want to rotate
	if (width < height) {
	    x1 = (int) (tempX1 + (centerX - (Math.cos(rad_angle)*(width/2))));
	    y1 = (int) (tempY1 + (centerY - (Math.sin(rad_angle)*(width/2))));
	    x2 = (int) (tempX2 + (centerX - (Math.cos(rad_angle)*(width/2))));
	    y2 = (int) (tempY2 + (centerY - (Math.sin(rad_angle)*(width/2))));
	} else {
	    x1 = (int) (tempX1 + (centerX - (Math.cos(rad_angle)*(height/2))));
	    y1 = (int) (tempY1 + (centerY - (Math.sin(rad_angle)*(height/2))));
		x2 = (int) (tempX2 + (centerX - (Math.cos(rad_angle)*(height/2))));
		y2 = (int) (tempY2 + (centerY - (Math.sin(rad_angle)*(height/2))));
	}
	
	linePoint1 = new Point(x1,y1);
	linePoint2 = new Point(x2,y2);
        
	Point[] points = new Point[2];
	points[0] = linePoint1;
	points[1] = linePoint2;
	return points;
    }     
	
	public void printFrame(int frame){
		if (frameData[frame]!=null)
			frameData[frame].printFrame(frame);	
	}

	public void printFrames(){
		System.out.println("Checking frames");
		for (int i=0;i<frameData.length;i++)
			printFrame(i);
	}
	
	public void setProperty(boolean ind, String property, String value,int id, int from, int to){
		for (int i=from;i<=to;i++)
			frameData[i].setProperty(ind,property,value,id);
		}
	}


