package xmlcaviargui;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import java.util.*;

/** 
 * Class used for loading picture onto the canvas.
 * @author Donald Nairn
 * @version 1.0
 */

public class DrawCanvas extends ImageCanvas {
 
    /** 
     * Points used to determine the top left corner of the rectangle,
     * and also the 2 endpoints of the line.
     */
    Point point1, point2, linePoint1 = null, linePoint2 = null;
   	
   	Body body=null;

    /** Field required to represent current knowledge about
     * the frame.
     */
    FrameData data;
    
    protected Icon image;

    /** Flag used to indicate whether there is an axis in this
     * box
     */
    private int index = -1;

    /**
     * Parameters required for creating new rectangle object.
     */
    int x = -1, y = -1 , width = -1, height = -1;
    
    private boolean activeBody=false;
    
    private boolean drawGaze=false;


	private boolean showNameTags=false;
    
	private boolean showCurrentSelectionTags=false;
    
	private boolean showAllBodyBoxes=false;

	private boolean showAllBodyTags=false;
	
	private boolean showNames=true;

    /**
     * Default constructor.
     */
    public DrawCanvas() {
	super();
    }

    /**
     * Constructor given an image object.
     */
    public DrawCanvas (Image image) {			
	super(image);
    }

    /** 
     * Used to draw the rectangles to the display
     * FrameData object required to update display consistently 
     * with current data.
     */
    public LinkedList drawBox (Point point1, Point point2, FrameData data) {
    this.activeBody=false;
    this.body=null;
	this.point1 = point1;
	this.point2 = point2;
	this.data = data;
	this.paint(getGraphics());
	// box only set if a box has been drawn
	if (x != -1 && y != -1 && width != -1 && height != -1) {
	    // screen has been redrawn to reflect new box drawn,
	    // however no box has been drawn so should not yet
	    // update box object properties
	    if (x == 0 && y == 0 && width == 0 && height == 0) {
		return new LinkedList();
	    }
	    LinkedList returnList = new LinkedList();
	    returnList.addLast(new Rectangle(x, y, width, height));
	    // Must be a single box and therefore have an axis if 
	    // both point are not null (as they are reset every time
	    if (linePoint1 != null && linePoint2 != null) {
		returnList.addLast(new Line2D.Float((float) linePoint1.getX(), (float) linePoint1.getY(),
						    (float) linePoint2.getX(), (float) linePoint2.getY()));
	    }
	    return returnList;
	} else return new LinkedList();
    }

	public void drawBody(Body body,FrameData data,boolean drawGaze){
//		this.point1=null;
//		this.point2=null;
		this.drawGaze=drawGaze;
		this.activeBody=true;
		this.body=body;
		this.data=data;
		this.paint(getGraphics());
	}
    
    /** 
     * Method to set the two points that make up the line of the axis
     * of orientation.
     */
    public void setAxisPoints() {
	int index = -1;
	int x1, y1, x2, y2;
	int centerX = x+width/2;
	int centerY = y+height/2;
	linePoint1 = null;
	linePoint2 = null;
	int angle = 0;
		
	for (int i = 0; i < data.singleBox.length; i++) {
	    if (data.singleBox[i].getSelected() == true) {
		index = i;
		angle = data.singleBox[index].getAxis();
		break;
	    }
	}
	
	for (int i = 0; i < data.groupBox.length; i++) {
	    if (data.groupBox[i].getSelected() == true) {
		index = i;
		angle = data.groupBox[index].getAxis();
		break;
	    }
	}
	
	if (index != -1) {

	    double rad_angle = (180-angle)* Math.PI / 180;
	    
	    // translate to center
	    // choose translation so fits in box,
	    // ie smallest of height or width
	    if (width <= height) {
		x1 = 0;
		y1 = 0;
		x2 = width;
		y2 = 0;
	    } else {
		x1 = 0;
		y1 = 0;
		x2 = height;
		y2 = 0;
	    }

	    // calculate rotation
	    int tempX1 = (int) ((Math.cos(rad_angle)*x1) + (-Math.sin(rad_angle)*y1));
	    int tempY1 = (int) ((Math.sin(rad_angle)*x1) + (Math.cos(rad_angle)*y1));
	    int tempX2 = (int) ((Math.cos(rad_angle)*x2) + (-Math.sin(rad_angle)*y2));
	    int tempY2 = (int) ((Math.sin(rad_angle)*x2) + (Math.cos(rad_angle)*y2));
 
	    // translate to point where we want to rotate
	    if (width < height) {
		x1 = (int) (tempX1 + (centerX - (Math.cos(rad_angle)*(width/2))));
		y1 = (int) (tempY1 + (centerY - (Math.sin(rad_angle)*(width/2))));
		x2 = (int) (tempX2 + (centerX - (Math.cos(rad_angle)*(width/2))));
		y2 = (int) (tempY2 + (centerY - (Math.sin(rad_angle)*(width/2))));
	    } else {
		x1 = (int) (tempX1 + (centerX - (Math.cos(rad_angle)*(height/2))));
		y1 = (int) (tempY1 + (centerY - (Math.sin(rad_angle)*(height/2))));
		x2 = (int) (tempX2 + (centerX - (Math.cos(rad_angle)*(height/2))));
		y2 = (int) (tempY2 + (centerY - (Math.sin(rad_angle)*(height/2))));
	    }

	    linePoint1 = new Point(x1,y1);
	    linePoint2 = new Point(x2,y2);
	}    
    }

    /**
     * Method to set the rectangle object that represents the box that
     * has been drawn on the canvas.
     */
    public void setBoxPoints() {
			
	boolean alreadyDrawn = false;
	boolean single = false;
	int index = -1;
	
	if (point2.x >= point1.x) {
	    x = point1.x;
	    width =  point2.x-point1.x;
	    if (point2.y >= point1.y) {
		y = point1.y;
		height = point2.y-point1.y;
	    } else {
		y = point2.y;
		height = point1.y-point2.y;
	    }
	} else {
	    x = point2.x;
	    width = point1.x-point2.x;
	    if (point2.y >= point1.y) {
		y = point1.y;
			    height = point2.y-point1.y;
	    } else {
		y = point2.y;
		height = point1.y-point2.y;
	    }
	}
    }

	public void drawBodyCircle(Graphics g,Circle circle,String label,Color color){
//		System.out.println("head is "+circle.getX()+" "+circle.getY()+" r= "+circle.getR());
		int xc=(circle.getX()-circle.getR())*CAVIARGUI.SCALE;
		int yc=(circle.getY()-circle.getR())*CAVIARGUI.SCALE;
		int r=circle.getR()*CAVIARGUI.SCALE;
		g.setColor(color);
		g.drawOval(xc,yc,r*2,r*2);
		g.drawString(label,xc,yc);
	}

	
	public void drawBody(Graphics g,Body body,Color color,boolean setNameTags,boolean drawGaze){
//		System.out.println("here "+color.toString()+" draw gaze "+drawGaze);
		if (body.isNull()==false){
			if (body.isHeadDraw()){
//				System.out.println("here head is draw");
				Circle head=body.getHead();
				if (drawGaze==false){
					if (setNameTags)	
						drawBodyCircle(g,head,"H",color);
					else
						drawBodyCircle(g,head,"",color);
				}
				int gaze=body.getGaze();
				if (gaze>=0){
					int x1=head.getX()*CAVIARGUI.SCALE;
					int y1=head.getY()*CAVIARGUI.SCALE;
					int r=head.getR()*CAVIARGUI.SCALE;	
					int x2=(int)(x1+(r+2)*Math.cos((360-gaze)*Math.PI/180));
					int y2=(int)(y1+(r+2)*Math.sin((360-gaze)*Math.PI/180));
					g.setColor(color);
//					System.out.println("drawing gaze as line from "+x1+" "+y1+" to "+x2+" "+y2+" color "+color.toString());	
					g.drawLine(x1,y1,x2,y2);
					if (setNameTags)
						g.drawString("G",x2,y2-5);
				}					
			}


			if (body.isShouldersDraw()){
				g.setColor(color);
				Line2D.Float sh=body.getShoulderLine();
				int x1=(int)sh.getX1()*CAVIARGUI.SCALE;
				int y1=(int)sh.getY1()*CAVIARGUI.SCALE;
				int x2=(int)sh.getX2()*CAVIARGUI.SCALE;
				int y2=(int)sh.getY2()*CAVIARGUI.SCALE;
				g.drawLine(x1,y1,x2,y2);
				if (setNameTags){
					g.drawString("R_S",x1,y1-5);
					g.drawString("L_S",x2,y2-5);
				}
			}
			if (body.isRightHandDraw())
				if (setNameTags)
					drawBodyCircle(g,body.getRighthand(),"R_H",color);
				else
					drawBodyCircle(g,body.getRighthand(),"",color);
			if (body.isLeftHandDraw())
				if (setNameTags)
					drawBodyCircle(g,body.getLefthand(),"L_H",color);
				else
					drawBodyCircle(g,body.getLefthand(),"",color);
			if (body.isRightFootDraw())
				if (setNameTags)			
					drawBodyCircle(g,body.getRightfoot(),"R_F",color);
				else	
					drawBodyCircle(g,body.getRightfoot(),"",color);
			if (body.isLeftFootDraw())
				if (setNameTags)			
					drawBodyCircle(g,body.getLeftfoot(),"L_F",color);
				else
					drawBodyCircle(g,body.getLeftfoot(),"",color);							
		}
	}

	public void drawStringforBox(Graphics g,String s,Rectangle rectangle){
		int x=(int)(rectangle.getX()+(rectangle.getWidth()/2))*CAVIARGUI.SCALE;
		int y=(int)((rectangle.getY()-5)*CAVIARGUI.SCALE);
		if ((rectangle.getY()-15)<0)
			y=(int)((rectangle.getY()+5)*CAVIARGUI.SCALE);
		g.drawString(s,x,y);
	}
	
    /** 
     * Overridden paint method to draw onto the canvas.
     */
    public void paint (Graphics g) {
	
	super.paint(g);
		
	if (data != null) {
	    
	    /* If the data has been initialised then we must draw the currently selected
	     * box along with the remaining boxes
	     */
	    
	    
	    /* Draw old boxes */
	    
	    Rectangle rectangle;
		
	    Line2D.Float line;
	    
	    if (data.singleBox != null) {
		
		g.setColor(Color.YELLOW);
		
		boolean singleselected=false;

		for (int i = 0; i < data.singleBox.length; i++) {
		    if (data.singleBox[i] != null && 
		    	data.singleBox[i].getSelected() == false &&	 
 				data.singleBox[i].getBoxDrawn() == true) {
			g.setColor(Color.YELLOW);
			rectangle = data.singleBox[i].getBox();
			g.drawRect((int)(rectangle.getX())*CAVIARGUI.SCALE,
				   (int)(rectangle.getY())*CAVIARGUI.SCALE,
				   (int)(rectangle.getWidth())*CAVIARGUI.SCALE,
				   (int)(rectangle.getHeight())*CAVIARGUI.SCALE);
			line = data.singleBox[i].getLine();
			g.drawLine((int)(line.getX1())*CAVIARGUI.SCALE,
				   (int)(line.getY1())*CAVIARGUI.SCALE,
				   (int)(line.getX2())*CAVIARGUI.SCALE,
				   (int)(line.getY2())*CAVIARGUI.SCALE);
			if (showNames)
				drawStringforBox(g,"S"+i,rectangle);
			Body body=data.singleBox[i].getBody();
			if (this.showAllBodyBoxes)
				drawBody(g,body,Color.YELLOW,this.showAllBodyTags,false);
		    }
		    if (data.singleBox[i].getSelected() == true){
		    	Color color=Color.RED;
		    	Body body_sel=data.singleBox[i].getBody();
		    	if (data.singleBox[i].getBoxDrawn()){
			    	rectangle=data.singleBox[i].getBox();
					if (showNames)
						drawStringforBox(g,"S"+i,rectangle);
		    	}		    	
		    	boolean show=(this.showCurrentSelectionTags) || (this.showAllBodyTags);
				drawBody(g,body_sel,color,show,false);
				
				singleselected=true;					
		    }
		    
		}
	    }
	    
	    if (data.groupBox != null) {
		
		g.setColor(Color.GREEN);
		
		for (int i = 0; i < data.groupBox.length; i++) {
		    if (data.groupBox[i] != null && 
		    data.groupBox[i].getSelected() == false && 
		    data.groupBox[i].getBoxDrawn() == true) {
			rectangle = data.groupBox[i].getBox();
			g.drawRect((int)(rectangle.getX())*CAVIARGUI.SCALE,
				   (int)(rectangle.getY())*CAVIARGUI.SCALE,
				   (int)(rectangle.getWidth())*CAVIARGUI.SCALE,
				   (int)(rectangle.getHeight())*CAVIARGUI.SCALE);
			line = data.groupBox[i].getLine();
			g.drawLine((int)(line.getX1())*CAVIARGUI.SCALE,
				   (int)(line.getY1())*CAVIARGUI.SCALE,
				   (int)(line.getX2())*CAVIARGUI.SCALE,
				   (int)(line.getY2())*CAVIARGUI.SCALE);
				if (showNames)
					drawStringforBox(g,"G"+i,rectangle);				   
		    }
		}
	    }
	    
	    /* Draw new box */
	    if (point1 != null || point2 != null) {
	    	if (this.activeBody)
	    		g.setColor(Color.red);
	    	else
				g.setColor(Color.orange);
			setBoxPoints();
			g.drawRect(x,y,width,height);
			setAxisPoints();
			if (linePoint1 != null && linePoint2 != null) {
			    g.drawLine((int) linePoint1.getX(), (int) linePoint1.getY(), (int) linePoint2.getX(), (int) linePoint2.getY());
			}
	    }
		
		if (this.body!=null){
			boolean prevtags=this.showNameTags;
			this.showNameTags=true;
//			System.out.println("In DRAW BODY in paint is "+this.body.getGaze());
			drawBody(g,this.body,Color.ORANGE,true,this.drawGaze);
			this.showNameTags=prevtags;	
//			System.out.println("after the call");			
		}else{
//			System.out.println("last call body was null");
		}
    }	
    }



	/**
	 * @param b
	 */
	public void setShowAllBodyBoxes() {
		if (showAllBodyBoxes)
			showAllBodyBoxes=false;
		else
			showAllBodyBoxes=true;
	}

	/**
	 * @param b
	 */
	public void setShowAllBodyTags() {
		if (showAllBodyTags)
			showAllBodyTags=false;
		else
			showAllBodyTags=true;
	}

	/**
	 * @param b
	 */
	public void setShowCurrentSelectionTags() {
		if (showCurrentSelectionTags)
			showCurrentSelectionTags=false;
		else
			showCurrentSelectionTags=true;
	}

	public void setBoxNames() {
		if (showNames)
			showNames=false;
		else
			showNames=true;
	}

}

