package xmlcaviargui;

import java.awt.*;
import java.awt.geom.*;

/**
 * Class for representing a single person box.
 */

public class SingleBox {
    
    private int axis;
    
    private Line2D.Float axisLine;
    
     /**
     * Parameters used to define where about the truths of that
     * name appear in the truths array.
     */
    public static final int APPEAR = 0, DISAPPEAR = 1, OCCLUDED = 2, /* TRACKED INDIVIDUAL*/
	WALK = 3, RUN  = 4, ACTIVE = 5, INACTIVE = 6, /* INDIVIDUALS MOVEMENTS */
	/* INDIVIDUALS ROLES */
	FIGHTER = 7, BROWSER = 8, LEFT_VICTIM = 9, LEAVING_GROUP = 10,
	WALKER = 11, LEFT_OBJECT = 12,
	
	/* SITUATIONS */
	MOVING_SIT = 13, INACTIVE_SIT = 14, BROWSING_SIT = 15, SHOPENTER_SIT=21,SHOPEXIT_SIT=22,
	
	/* SCENARIOS */ 
	BROWSING = 16, IMMOBILE = 17, WALKING = 18, DROP_DOWN = 19, 
	WINDOWSHOP_SCE=23,SHOPENTER_SCE=24,SHOPEXIT_SCE=25,SHOPREENTER_SCE=26,
	
	/* EVENTS */
	PERSON_IMMOBILE = 20, 
	 
	/*DEFAULT*/
	NONE_ROLE=27, NONE_SIT=28,NONE_SCE=29, NONE_MOV=30;


    /**This is used for mapping, and if not kept up to date when adding to the above will cause
      *the ChangeBoxSequence program to function incorrectly
      */

    public static String[] text = {"appear", "disappear", "occluded", "walking", "running", "active",
	"inactive", "fighter", "browser", "leaving victim", "leaving group", "walker",
	"leaving object", "moving", "inactive", "browsing", "browsing", "immobile", 
	"walking", "drop down", "immobile", "shop enter", "shop exit", "windowshop", 
	"shop enter", "shop exit", "shop reenter", "none", "none", "none", "none"};
    
    public static int[] roles = {FIGHTER, BROWSER, LEFT_VICTIM, LEAVING_GROUP, WALKER,
	LEFT_OBJECT, NONE_ROLE};

    public static int[] scenarios = {BROWSING, IMMOBILE, WALKING, DROP_DOWN, WINDOWSHOP_SCE, 
	SHOPENTER_SCE, SHOPEXIT_SCE, SHOPREENTER_SCE, NONE_SCE};

    public static int[] situations = {MOVING_SIT, INACTIVE_SIT, BROWSING_SIT, SHOPENTER_SIT, 
	SHOPEXIT_SIT, NONE_SIT};

    public static int[] movements = {WALK, RUN, ACTIVE, INACTIVE, NONE_MOV};
	
    /**
     * field for length of ground truths array.
     */
     public static final int LENGTH = 31;

    /**
     *
     */
    public static final int MOVEMENT_LENGTH = 5;

    /**
     *
     */
    public static final int ROLE_LENGTH = 7;
    
    /**
     *
     */
    public static final int SIT_LENGTH = 6;
    
    /**
     *
     */
    public static final int SCE_LENGTH = 9;        

    /**
     * Truths array for this SingleBox object.
     */
    public boolean truths[] = new boolean[LENGTH];

    private boolean boxDrawn;

    private boolean selected;
    
    private Rectangle box;
    
    private Body body;   


    /** 
     * Default constructor.
     */
    public SingleBox() {
	for (int i = 0; i < truths.length; i++) {
	    truths[i] = false;
	}
	// default inactive
	truths[NONE_SIT] = true;
	truths[NONE_ROLE] = true;
	truths[NONE_SCE] = true;
	truths[NONE_MOV] = true;
	axis = 90;
	boxDrawn = false;
	selected = false;
	box = null;
	axisLine = new Line2D.Float();
	body = new Body();
    }

   /** 
     * Checks if the singlebox has the default values in all the fields.
     */

    public boolean isDefault(){
    boolean empty=true;
	for (int i = 0; i < truths.length; i++) 
		if ((i!=NONE_MOV) && (i!=NONE_SIT) &&(i!=NONE_ROLE) &&(i!=NONE_SCE) &&(truths[i]!=false))
			empty=false;
	if (truths[NONE_MOV]!=true)
		empty=false;
	if (truths[NONE_SIT]!=true)
		empty=false;
	if (truths[NONE_ROLE]!=true)
		empty=false;
	if (truths[NONE_SCE]!=true)
		empty=false;				
	if (axis!=90)
		empty=false;
	if (boxDrawn!=false)
		empty=false;
	if (box!=null)
		empty=false;
	return empty;		
	}
   
    
	/**
	 * Constructor specifying all relavent parameters.
	 */
	public SingleBox(boolean[] truths, Rectangle box, int axis, Line2D.Float axisLine,Body body) {
	//this.truths = truths;
	for (int i = 0; i < truths.length; i++){
		this.truths[i] = truths[i];
	}
	this.box = box;
	this.axis = axis;
	this.axisLine = axisLine;
	this.body = body;
	boxDrawn = true;
	}    


	public String CheckRoles(){
	int j=0;
	String Roles="";
	for (int i=0;i<roles.length;i++)
		if (truths[roles[i]]==true){
			j++;
			Roles+=text[roles[i]]+" , ";
			}
		if (j<=1) 
			Roles=null; 
		return Roles;	
	}

	public String CheckSituations(){
	int j=0;
	String Situations="";
	for (int i=0;i<situations.length;i++)
		if (truths[situations[i]]==true){
			j++;
			Situations+=text[situations[i]]+" ,  ";
			}
		if (j<=1) 
			Situations=null; 
		return Situations;	
	}

	public String CheckScenarios(){
	int j=0;
	String Scenarios="";
	for (int i=0;i<scenarios.length;i++)
		if (truths[scenarios[i]]==true){
			j++;
			Scenarios+=text[scenarios[i]]+" , ";
			}
		if (j<=1) 
			Scenarios=null; 
		return Scenarios;	
	}
	
	public void setRol(String rol){
		System.out.println("Setting rol to "+rol);
		for (int i=0;i<roles.length;i++)
			if (rol.equals(text[roles[i]])==true){
				System.out.println("FOUND!");
				for (int j=0;j<roles.length;j++)
					truths[roles[j]]=false;			
				truths[roles[i]]=true;
				}
	}
		
	public void setSituation(String sit){
		System.out.println("Setting situation to "+sit);
		for (int i=0;i<situations.length;i++)
			if (sit.equals(text[situations[i]])==true){
				System.out.println("FOUND!");
				for (int j=0;j<situations.length;j++)
					truths[situations[j]]=false;			
				truths[situations[i]]=true;
				}
	}	
	
	public void setScenario(String sce){
		System.out.println("Setting scenario to "+sce);
		for (int i=0;i<scenarios.length;i++)
			if (sce.equals(text[scenarios[i]])==true){
				System.out.println("FOUND!");
				for (int j=0;j<scenarios.length;j++)
					truths[scenarios[j]]=false;			
				truths[scenarios[i]]=true;
				}
	}

	/**
	 * @return Returns the body.
	 */
	public Body getBody() {
		return body;
	}
	/**
	 * @param body The body to set.
	 */
	public void setBody(Body body) {
		this.body = body;
	}
	public boolean isBodyEmpty(){
		return body.isNull();
	}
	
    /** 
     * Get method for GUI to get value of appear
     * value in truths array. 
     */
    public boolean getAppear() {
	return truths[APPEAR];	
    }

    /** 
     * Get method for GUI to get value of disappear
     * value in truths array. 
     */
    public boolean getDisappear() {
	return truths[DISAPPEAR];
    }

    public boolean getOccluded() {
	return truths[OCCLUDED];
    }

    /** 
     * Get method for GUI to get value of walk
     * value in truths array. 
     */
    public boolean getWalk() {
	return truths[WALK];
    }

    /** 
     * Get method for GUI to get value of run
     * value in truths array. 
     */
    public boolean getRun() {
	return truths[RUN];
    }

    /** 
     * Get method for GUI to get value of active
     * value in truths array. 
     */
    public boolean getActive() {
	return truths[ACTIVE];
    }

    /** 
     * Get method for GUI to get value of inactive
     * value in truths array. 
     */
    public boolean getInActive() {
	return truths[INACTIVE];
    }

     /** 
     * Get method for GUI to get value of none rol
     * value in truths array. 
     */
    public boolean getNoneRol() {
	return truths[NONE_ROLE];
    }

     /** 
     * Get method for GUI to get value of none sit
     * value in truths array. 
     */
    public boolean getNoneSit() {
	return truths[NONE_SIT];
    }

     /** 
     * Get method for GUI to get value of none sce
     * value in truths array. 
     */
    public boolean getNoneSce() {
	return truths[NONE_SCE];
    }
    
     /** 
     * Get method for GUI to get value of none sce
     * value in truths array. 
     */
    public boolean getNoneMov() {
	return truths[NONE_MOV];
    }
    
     /** 
     * Get method for GUI to get value of fighter
     * value in truths array. 
     */
    public boolean getFighter() {
	return truths[FIGHTER];
    }

    /** 
     * Get method for GUI to get value of browser
     * value in truths array. 
     */
    public boolean getBrowser() {
	return truths[BROWSER];
    }

    /** 
     * Get method for GUI to get value of left behind
     * value in truths array. 
     */
    public boolean getLeftBehind() {
	return truths[LEFT_VICTIM];
    }

    /** 
     * Get method for GUI to get value of leaving group
     * value in truths array. 
     */
    public boolean getLeavingGroup() {
	return truths[LEAVING_GROUP];
    }

    /** 
     * Get method for GUI to get value of walker
     * value in truths array. 
     */
    public boolean getWalker() {
	return truths[WALKER];
    }

    /** 
     * Get method for GUI to get value of left object
     * value in truths array. 
     */
    public boolean getLeftObject() {
	return truths[LEFT_OBJECT];
    }

    /** 
     * Get method for GUI to get value of moving situation
     * value in truths array. 
     */
    public boolean getMovingSit() {
	return truths[MOVING_SIT];
    }

    /** 
     * Get method for GUI to get value of shop enter situation
     * value in truths array. 
     */
    public boolean getShopenterSit() {
	return truths[SHOPENTER_SIT];
    }

    /** 
     * Get method for GUI to get value of shop exit situation
     * value in truths array. 
     */
    public boolean getShopexitSit() {
	return truths[SHOPEXIT_SIT];
    }

    /** 
     * Get method for GUI to get value of window shop scenario
     * value in truths array. 
     */
    public boolean getWindowshopSce() {
	return truths[WINDOWSHOP_SCE];
    }
    
    /** 
     * Get method for GUI to get value of shop enter scenario
     * value in truths array. 
     */
    public boolean getShopenterSce() {
	return truths[SHOPENTER_SCE];
    }

    /** 
     * Get method for GUI to get value of shop exit scenario
     * value in truths array. 
     */
    public boolean getShopexitSce() {
	return truths[SHOPEXIT_SCE];
    }

    /** 
     * Get method for GUI to get value of shop reenter scenario
     * value in truths array. 
     */
    public boolean getShopreenterSce() {
	return truths[SHOPREENTER_SCE];
    }
        
    /** 
     * Get method for GUI to get value of inactive situation
     * value in truths array. 
     */
    public boolean getInactiveSit() {
	return truths[INACTIVE_SIT];
    }

    /** 
     * Get method for GUI to get value of browsing situation
     * value in truths array. 
     */
    public boolean getBrowsingSit() {
	return truths[BROWSING_SIT];
    }

    /** 
     * Get method for GUI to get value of browsing
     * value in truths array. 
     */
    public boolean getBrowsing() {
	return truths[BROWSING];
    }

    /** 
     * Get method for GUI to get value of immobile
     * value in truths array. 
     */
    public boolean getImmobile() {
	return truths[IMMOBILE];
    }

    /** 
     * Get method for GUI to get value of immobile
     * value in truths array. 
     */
    public boolean getWalking() {
	return truths[WALKING];
    }

    /** 
     * Get method for GUI to get value of immobile
     * value in truths array. 
     */
    public boolean getDropDown() {
	return truths[DROP_DOWN];
    }

    /** 
     * Get method for GUI to get value of immobile
     * value in truths array. 
     */
    public boolean getPersonImmobile() {
	return truths[PERSON_IMMOBILE];
    }

    /**
     * Get method for GUI to get whether a box 
     * has been drawn asscociated with this SingleBox object.
     */
    public boolean getBoxDrawn() {
	return boxDrawn;
    }

    /**
     * Get method for GUI to get whether a this SingleBox object.
     * has been selected by the user.
     */
    public boolean getSelected() {
	return selected;
    }

    /**
     * get method for the GUI to return the Rectangle object associated
     * with this SingleBox object.
     */
    public Rectangle getBox() {
	return box;
    }

    /**
     * Returns the axis of orientation assciated with this SingleBox object.
     */
    public int getAxis() {
	return axis;
    }

    /**
     * Returns the line of the orientation worked out from the
     * Rectangle object and the axis parameter if this SingleBox object.
     */
    public Line2D.Float getLine() {
	return axisLine;
    }

    /**
     * Returns all the truths asscoiated with this SingleBox object.
     */
    public boolean[] getTruths() {
	return truths;
    }

    /** 
     * Toggles the value of the appear attribute of this SingleBox object.
     */
    public void setAppear() {
	truths[APPEAR] = !truths[APPEAR];
	if (truths[APPEAR] == true) {
	    if (truths[APPEAR] == truths[DISAPPEAR]) {
		setDisappear();
	    }
	    if (truths[APPEAR] == truths[OCCLUDED]) {
		setOccluded();
	    }
	}
    }

    /** 
     * Toggles the value of the disappear attribute of this SingleBox object.
     */
    public void setDisappear() {
	truths[DISAPPEAR] = !truths[DISAPPEAR];
	if (truths[DISAPPEAR] == true) {
	    if (truths[DISAPPEAR] == truths[APPEAR]) {
		setAppear();
	    } 
	    if (truths[DISAPPEAR] == truths[OCCLUDED]) {
		setOccluded();
	    }
	}
    }

    public void setOccluded() {
	truths[OCCLUDED] = !truths[OCCLUDED];
	if (truths[OCCLUDED] == true) {
	    if (truths[OCCLUDED] == truths[APPEAR]) {
		setAppear();
	    } 
	    if (truths[OCCLUDED] == truths[DISAPPEAR]) {
		setDisappear();
	    }
	}
    }


    /** 
     * Toggles the value of the walk attribute of this SingleBox object.
     */
    public void setWalk() {
	truths[WALK] = !truths[WALK];
	if (truths[WALK] == true) {
	    if(truths[WALK] == truths[NONE_MOV]){
		setNoneMov();
	    }
	    if (truths[WALK] == truths[ACTIVE]) {
		setActive();
	    } 
	    if (truths[WALK] == truths[INACTIVE]) {
		setInActive();
	    }
	    if (truths[WALK] == truths[RUN]) {
		setRun();
	    }
	}
    }
    
    /** 
     * Toggles the value of the run attribute of this SingleBox object.
     */
    public void setRun() {
	truths[RUN] = !truths[RUN];
	if (truths[RUN] == true) {
	    if (truths[RUN] == truths[ACTIVE]) {
		setActive();
	    }
	    if (truths[RUN] == truths[INACTIVE]) {
		setInActive();
	    }
	    if (truths[RUN] == truths[WALK]) {
		setWalk();
	    }
	    if(truths[RUN] == truths[NONE_MOV]){
		setNoneMov();
	    }
	}
    }
    
    /** 
     * Toggles the value of the active attribute of this SingleBox object.
     */
    public void setActive() {
	truths[ACTIVE] = !truths[ACTIVE];
	if (truths[ACTIVE] == true) {
	    if (truths[ACTIVE] == truths[WALK]) {
		setWalk();
	    }
	    if (truths[ACTIVE] == truths[INACTIVE]) {
		setInActive();
	    } 
	    if (truths[ACTIVE] == truths[RUN]) {
		setRun();
	    }
	    if(truths[ACTIVE] == truths[NONE_MOV]){
		setNoneMov();
	    }
	}
    }

    /** 
     * Toggles the value of the inactive attribute of this SingleBox object.
     */
    public void setInActive() {
	truths[INACTIVE] = !truths[INACTIVE];
	if (truths[INACTIVE] == true) {
	    if (truths[INACTIVE] == truths[ACTIVE]) {
		setActive();
	    } 
	    if (truths[INACTIVE] == truths[WALK]) {
		setWalk();
	    }
	    if (truths[INACTIVE] == truths[RUN]) {
		setRun();
	    }
	    if(truths[INACTIVE] == truths[NONE_MOV]){
		setNoneMov();
	    }
	}
    }

    /** 
     * Toggles the value of the none movement attribute of this SingleBox object.
     */
    public void setNoneMov() {
	truths[NONE_MOV] = !truths[NONE_MOV];
	if (truths[INACTIVE] == true) {
	    if (truths[NONE_MOV] == truths[ACTIVE]) {
		setActive();
	    } 
	    if (truths[NONE_MOV] == truths[WALK]) {
		setWalk();
	    }
	    if (truths[NONE_MOV] == truths[RUN]) {
		setRun();
	    }
	    if(truths[NONE_MOV] == truths[INACTIVE]){
		setInActive();
	    }
	}
    }
    /** 
     * Toggles the value of the fighter attribute of this SingleBox object.
     */
    public void setNoneRol() {
	truths[NONE_ROLE] = !truths[NONE_ROLE];
	if (truths[NONE_ROLE] == true) {
	    if (truths[NONE_ROLE] == truths[FIGHTER]) {
		setFighter();
	    } 	
	    if (truths[NONE_ROLE] == truths[BROWSER]) {
		setBrowser();
	    } 
	    if (truths[NONE_ROLE] == truths[LEFT_VICTIM]) {
		setLeftBehind();
	    }
	    if (truths[NONE_ROLE] == truths[LEAVING_GROUP]) {
		setLeavingGroup();
	    }
	    if (truths[NONE_ROLE] == truths[WALKER]) {
		setWalker();
	    }
	    if (truths[NONE_ROLE] == truths[LEFT_OBJECT]) {
		setLeftObject();
	    }
	}
	System.out.println("En setfighter "+truths[BROWSER]);
    }


    /** 
     * Toggles the value of the fighter attribute of this SingleBox object.
     */
    public void setFighter() {
	truths[FIGHTER] = !truths[FIGHTER];
	if (truths[FIGHTER] == true) {
	    if (truths[FIGHTER] == truths[NONE_ROLE]) {
		setNoneRol();
	    } 
	    if (truths[FIGHTER] == truths[BROWSER]) {
		setBrowser();
	    } 
	    if (truths[FIGHTER] == truths[LEFT_VICTIM]) {
		setLeftBehind();
	    }
	    if (truths[FIGHTER] == truths[LEAVING_GROUP]) {
		setLeavingGroup();
	    }
	    if (truths[FIGHTER] == truths[WALKER]) {
		setWalker();
	    }
	    if (truths[FIGHTER] == truths[LEFT_OBJECT]) {
		setLeftObject();
	    }
	}
	System.out.println("En setfighter "+truths[BROWSER]);
    }

    /** 
     * Toggles the value of the browser attribute of this SingleBox object.
     */
    public void setBrowser() {
	truths[BROWSER] = !truths[BROWSER];
	if (truths[BROWSER] == true) {
	    if (truths[BROWSER] == truths[NONE_ROLE]) {
		setNoneRol();
	    }
	    if (truths[BROWSER] == truths[FIGHTER]) {
	    System.out.println("set browser llama a setfighter");
		setFighter();
	    } 
	    if (truths[BROWSER] == truths[LEFT_VICTIM]) {
		setLeftBehind();
	    }
	    if (truths[BROWSER] == truths[LEAVING_GROUP]) {
		setLeavingGroup();
	    }
	    if (truths[BROWSER] == truths[WALKER]) {
		setWalker();
	    }
	    if (truths[BROWSER] == truths[LEFT_OBJECT]) {
		setLeftObject();
	    }
	}
    }
    
    /** 
     * Toggles the value of the left behind attribute of this SingleBox object.
     */
    public void setLeftBehind() {
	truths[LEFT_VICTIM] = !truths[LEFT_VICTIM];
	if (truths[LEFT_VICTIM] == true) {
	    if (truths[LEFT_VICTIM] == truths[NONE_ROLE]) {
		setNoneRol();
	    } 
	    if (truths[LEFT_VICTIM] == truths[FIGHTER]) {
		setFighter();
	    } 
	    if (truths[LEFT_VICTIM] == truths[BROWSER]) {
		setBrowser();
	    }
	    if (truths[LEFT_VICTIM] == truths[LEAVING_GROUP]) {
		setLeavingGroup();
	    }
	    if (truths[LEFT_VICTIM] == truths[WALKER]) {
		setWalker();
	    }
	    if (truths[LEFT_VICTIM] == truths[LEFT_OBJECT]) {
		setLeftObject();
	    }
	}
	
    }

    /** 
     * Toggles the value of the leaving group attribute of this SingleBox object.
     */
    public void setLeavingGroup() {
	truths[LEAVING_GROUP] = !truths[LEAVING_GROUP];
	if (truths[LEAVING_GROUP] == true) {
	    if (truths[LEAVING_GROUP] == truths[NONE_ROLE]) {
		setNoneRol();
	    } 	
	    if (truths[LEAVING_GROUP] == truths[FIGHTER]) {
		setFighter();
	    } 
	    if (truths[LEAVING_GROUP] == truths[BROWSER]) {
		setBrowser();
	    }
	    if (truths[LEAVING_GROUP] == truths[LEFT_VICTIM]) {
		setLeftBehind();
	    }
	    if (truths[LEAVING_GROUP] == truths[WALKER]) {
		setWalker();
	    }
	    if (truths[LEAVING_GROUP] == truths[LEFT_OBJECT]) {
		setLeftObject();
	    }
	}
    }

    /** 
     * Toggles the value of the walker attribute of this SingleBox object.
     */
    public void setWalker() {
	truths[WALKER] = !truths[WALKER];
	if (truths[WALKER] == true) {
	    if (truths[WALKER] == truths[NONE_ROLE]) {
		setNoneRol();
	    } 
	    if (truths[WALKER] == truths[FIGHTER]) {
		setFighter();
	    } 
	    if (truths[WALKER] == truths[BROWSER]) {
		setBrowser();
	    }
	    if (truths[WALKER] == truths[LEFT_VICTIM]) {
		setLeftBehind();
	    }
	    if (truths[WALKER] == truths[LEAVING_GROUP]) {
		setLeavingGroup();
	    }
	    if (truths[WALKER] == truths[LEFT_OBJECT]) {
		setLeftObject();
	    }
	}
    }

     /** 
     * Toggles the value of the left object attribute of this SingleBox object.
     */
    public void setLeftObject() {
	truths[LEFT_OBJECT] = !truths[LEFT_OBJECT];
	if (truths[LEFT_OBJECT] == true) {
	    if (truths[LEFT_OBJECT] == truths[NONE_ROLE]) {
		setNoneRol();
	    } 
	    if (truths[LEFT_OBJECT] == truths[FIGHTER]) {
		setFighter();
	    } 
	    if (truths[LEFT_OBJECT] == truths[BROWSER]) {
		setBrowser();
	    }
	    if (truths[LEFT_OBJECT] == truths[LEFT_VICTIM]) {
		setLeftBehind();
	    }
	    if (truths[LEFT_OBJECT] == truths[LEAVING_GROUP]) {
		setLeavingGroup();
	    }
	    if (truths[LEFT_OBJECT] == truths[WALKER]) {
		setWalker();
	    }
	}
   }

    /** 
     * Toggles the value of the none situation attribute of this SingleBox object.
     */
    public void setNoneSit() {
	truths[NONE_SIT] = !truths[NONE_SIT];
	if (truths[NONE_SIT] == true) {
		    if (truths[NONE_SIT] == truths[MOVING_SIT]) {
			setMovingSit();
		    } 
		    if (truths[NONE_SIT] == truths[INACTIVE_SIT]) {
			setInactiveSit();
		    } 
		    if (truths[NONE_SIT] == truths[BROWSING_SIT]) {
			setBrowsingSit();
		    }
		    if (truths[NONE_SIT] == truths[SHOPENTER_SIT]) {
			setShopenterSit();
		    }
		    if (truths[NONE_SIT] == truths[SHOPEXIT_SIT]) {
			setShopexitSit();
		    }
		}
    }
    
    
    /** 
     * Toggles the value of the moving situation attribute of this SingleBox object.
     */
    public void setMovingSit() {
	truths[MOVING_SIT] = !truths[MOVING_SIT];
	if (truths[MOVING_SIT] == true) {
		    if (truths[MOVING_SIT] == truths[NONE_SIT]) {
			setNoneSit();
		    } 
		    if (truths[MOVING_SIT] == truths[INACTIVE_SIT]) {
			setInactiveSit();
		    } 
		    if (truths[MOVING_SIT] == truths[BROWSING_SIT]) {
			setBrowsingSit();
		    }
		    if (truths[MOVING_SIT] == truths[SHOPENTER_SIT]) {
			setShopenterSit();
		    }
		    if (truths[MOVING_SIT] == truths[SHOPEXIT_SIT]) {
			setShopexitSit();
		    }
		}	
    }

    /** 
     * Toggles the value of the shop enter situation attribute of this SingleBox object.
     */
    public void setShopenterSit() {
	truths[SHOPENTER_SIT] = !truths[SHOPENTER_SIT];
	if (truths[SHOPENTER_SIT] == true) {
		    if (truths[SHOPENTER_SIT] == truths[NONE_SIT]) {
			setNoneSit();
		    } 
		    if (truths[SHOPENTER_SIT] == truths[MOVING_SIT]) {
			setMovingSit();
		    } 
		    if (truths[SHOPENTER_SIT] == truths[INACTIVE_SIT]) {
			setInactiveSit();
		    }
		    if (truths[SHOPENTER_SIT] == truths[BROWSING_SIT]) {
			setBrowsingSit();
		    }
		    if (truths[SHOPENTER_SIT] == truths[SHOPEXIT_SIT]) {
			setShopexitSit();
		    }
		}	
    }

    /** 
     * Toggles the value of the shop exit situation attribute of this SingleBox object.
     */
    public void setShopexitSit() {
	truths[SHOPEXIT_SIT] = !truths[SHOPEXIT_SIT];
	if (truths[SHOPEXIT_SIT] == true) {
		    if (truths[SHOPEXIT_SIT] == truths[NONE_SIT]) {
			setNoneSit();
		    } 
		    if (truths[SHOPEXIT_SIT] == truths[MOVING_SIT]) {
			setMovingSit();
		    } 
		    if (truths[SHOPEXIT_SIT] == truths[INACTIVE_SIT]) {
			setInactiveSit();
		    }
		    if (truths[SHOPEXIT_SIT] == truths[BROWSING_SIT]) {
			setBrowsingSit();
		    }
		    if (truths[SHOPEXIT_SIT] == truths[SHOPENTER_SIT]) {
			setShopenterSit();
		    }
		}	
    }

    /** 
     * Toggles the value of the window shop scenario attribute of this SingleBox object.
     */
    public void setWindowshopSce() {
	truths[WINDOWSHOP_SCE] = !truths[WINDOWSHOP_SCE];
	if (truths[WINDOWSHOP_SCE] == true) {
		    if (truths[WINDOWSHOP_SCE] == truths[NONE_SCE]) {
			setNoneSce();
		    } 
		    if (truths[WINDOWSHOP_SCE] == truths[BROWSING]) {
			setBrowsing();
		    } 
		    if (truths[WINDOWSHOP_SCE] == truths[IMMOBILE]) {
			setImmobile();
		    }
		    if (truths[WINDOWSHOP_SCE] == truths[WALKING]) {
			setWalking();
		    }
		    if (truths[WINDOWSHOP_SCE] == truths[DROP_DOWN]) {
			setDropDown();
		    }
		    if (truths[WINDOWSHOP_SCE] == truths[SHOPENTER_SCE]) {
			setShopenterSce();
		    }
		    if (truths[WINDOWSHOP_SCE] == truths[SHOPEXIT_SCE]) {
			setShopexitSce();
		    }
		    if (truths[WINDOWSHOP_SCE] == truths[SHOPREENTER_SCE]) {
			setShopreenterSce();
		    }
		}		

    }
    
    /** 
     * Toggles the value of the shop enter scenario attribute of this SingleBox object.
     */
    public void setShopenterSce() {
	truths[SHOPENTER_SCE] = !truths[SHOPENTER_SCE];
	if (truths[SHOPENTER_SCE] == true) {
		    if (truths[SHOPENTER_SCE] == truths[NONE_SCE]) {
			setNoneSce();
		    } 
		    if (truths[SHOPENTER_SCE] == truths[BROWSING]) {
			setBrowsing();
		    } 
		    if (truths[SHOPENTER_SCE] == truths[IMMOBILE]) {
			setImmobile();
		    }
		    if (truths[SHOPENTER_SCE] == truths[WALKING]) {
			setWalking();
		    }
		    if (truths[SHOPENTER_SCE] == truths[DROP_DOWN]) {
			setDropDown();
		    }
		    if (truths[SHOPENTER_SCE] == truths[WINDOWSHOP_SCE]) {
			setWindowshopSce();
		    }
		    if (truths[SHOPENTER_SCE] == truths[SHOPEXIT_SCE]) {
			setShopexitSce();
		    }
		    if (truths[SHOPENTER_SCE] == truths[SHOPREENTER_SCE]) {
			setShopreenterSce();
		    }
		}		
    }

    /** 
     * Toggles the value of the shop exit scenario attribute of this SingleBox object.
     */
    public void setShopexitSce() {
	truths[SHOPEXIT_SCE] = !truths[SHOPEXIT_SCE];
	if (truths[SHOPEXIT_SCE] == true) {
		    if (truths[SHOPEXIT_SCE] == truths[NONE_SCE]) {
			setNoneSce();
		    } 
		    if (truths[SHOPEXIT_SCE] == truths[BROWSING]) {
			setBrowsing();
		    } 
		    if (truths[SHOPEXIT_SCE] == truths[IMMOBILE]) {
			setImmobile();
		    }
		    if (truths[SHOPEXIT_SCE] == truths[WALKING]) {
			setWalking();
		    }
		    if (truths[SHOPEXIT_SCE] == truths[DROP_DOWN]) {
			setDropDown();
		    }
		    if (truths[SHOPEXIT_SCE] == truths[SHOPENTER_SCE]) {
			setShopenterSce();
		    }
		    if (truths[SHOPEXIT_SCE] == truths[WINDOWSHOP_SCE]) {
			setWindowshopSce();
		    }
		    if (truths[SHOPEXIT_SCE] == truths[SHOPREENTER_SCE]) {
			setShopreenterSce();
		    }
		}			
    }

    /** 
     * Toggles the value of the shop reenter scenario attribute of this SingleBox object.
     */
    public void setShopreenterSce() {
	truths[SHOPREENTER_SCE] = !truths[SHOPREENTER_SCE];
//	System.out.println("setting shop reenter to "+truths[SHOPREENTER_SCE]);
	if (truths[SHOPREENTER_SCE] == true) {
		    if (truths[SHOPREENTER_SCE] == truths[NONE_SCE]) {
			setNoneSce();
		    } 
		    if (truths[SHOPREENTER_SCE] == truths[BROWSING]) {
			setBrowsing();
		    } 
		    if (truths[SHOPREENTER_SCE] == truths[IMMOBILE]) {
			setImmobile();
		    }
		    if (truths[SHOPREENTER_SCE] == truths[WALKING]) {
			setWalking();
		    }
		    if (truths[SHOPREENTER_SCE] == truths[DROP_DOWN]) {
			setDropDown();
		    }
		    if (truths[SHOPREENTER_SCE] == truths[SHOPEXIT_SCE]) {
			setShopexitSce();
		    }
		    if (truths[SHOPREENTER_SCE] == truths[WINDOWSHOP_SCE]) {
			setWindowshopSce();
		    }
		    if (truths[SHOPREENTER_SCE] == truths[SHOPENTER_SCE]) {
			setShopenterSce();
		    }
		}				
    }
 
    /** 
     * Toggles the value of the inactive attribute of this SingleBox object.
     */
    public void setInactiveSit() {
	truths[INACTIVE_SIT] = !truths[INACTIVE_SIT];
	if (truths[INACTIVE_SIT] == true) {
		    if (truths[INACTIVE_SIT] == truths[NONE_SIT]) {
			setNoneSit();
		    } 
		    if (truths[INACTIVE_SIT] == truths[MOVING_SIT]) {
			setMovingSit();
		    } 
		    if (truths[INACTIVE_SIT] == truths[BROWSING_SIT]) {
			setBrowsingSit();
		    }
		    if (truths[INACTIVE_SIT] == truths[SHOPENTER_SIT]) {
			setShopenterSit();
		    }
		    if (truths[INACTIVE_SIT] == truths[SHOPEXIT_SIT]) {
			setShopexitSit();
		    }
		}	
    }

    /** 
     * Toggles the value of the browsing situation attribute of this SingleBox object.
     */
    public void setBrowsingSit() {
	truths[BROWSING_SIT] = !truths[BROWSING_SIT];
	if (truths[BROWSING_SIT] == true) {
		    if (truths[BROWSING_SIT] == truths[NONE_SIT]) {
			setNoneSit();
		    } 
		    if (truths[BROWSING_SIT] == truths[MOVING_SIT]) {
			setMovingSit();
		    } 
		    if (truths[BROWSING_SIT] == truths[INACTIVE_SIT]) {
			setInactiveSit();
		    }
		    if (truths[BROWSING_SIT] == truths[SHOPENTER_SIT]) {
			setShopenterSit();
		    }
		    if (truths[BROWSING_SIT] == truths[SHOPEXIT_SIT]) {
			setShopexitSit();
		    }
		}	
    }

    /** 
     * Toggles the value of the none scenario attribute of this SingleBox object.
     */
    public void setNoneSce() {
	truths[NONE_SCE] = !truths[NONE_SCE];
	if (truths[NONE_SCE] == true) {
		    if (truths[NONE_SCE] == truths[BROWSING]) {
			setBrowsing();
		    } 
		    if (truths[NONE_SCE] == truths[IMMOBILE]) {
			setImmobile();
		    } 
		    if (truths[NONE_SCE] == truths[WALKING]) {
			setWalking();
		    }
		    if (truths[NONE_SCE] == truths[DROP_DOWN]) {
			setDropDown();
		    }
		    if (truths[NONE_SCE] == truths[WINDOWSHOP_SCE]) {
			setWindowshopSce();
		    }
		    if (truths[NONE_SCE] == truths[SHOPENTER_SCE]) {
			setShopenterSce();
		    }
		    if (truths[NONE_SCE] == truths[SHOPEXIT_SCE]) {
			setShopexitSce();
		    }
		    if (truths[NONE_SCE] == truths[SHOPREENTER_SCE]) {
			setShopreenterSce();
		    }
		}	
	
    }
    
    /** 
     * Toggles the value of the browsing attribute of this SingleBox object.
     */
    public void setBrowsing() {
	truths[BROWSING] = !truths[BROWSING];
	if (truths[BROWSING] == true) {
		    if (truths[BROWSING] == truths[NONE_SCE]) {
			setNoneSce();
		    } 
		    if (truths[BROWSING] == truths[IMMOBILE]) {
			setImmobile();
		    } 
		    if (truths[BROWSING] == truths[WALKING]) {
			setWalking();
		    }
		    if (truths[BROWSING] == truths[DROP_DOWN]) {
			setDropDown();
		    }
		    if (truths[BROWSING] == truths[WINDOWSHOP_SCE]) {
			setWindowshopSce();
		    }
		    if (truths[BROWSING] == truths[SHOPENTER_SCE]) {
			setShopenterSce();
		    }
		    if (truths[BROWSING] == truths[SHOPEXIT_SCE]) {
			setShopexitSce();
		    }
		    if (truths[BROWSING] == truths[SHOPREENTER_SCE]) {
			setShopreenterSce();
		    }
		}	
    }

     /** 
     * Toggles the value of the immobile attribute of this SingleBox object.
     */
    public void setImmobile() {
	truths[IMMOBILE] = !truths[IMMOBILE];
	if (truths[IMMOBILE] == true) {
		    if (truths[IMMOBILE] == truths[NONE_SCE]) {
			setNoneSce();
		    } 
		    if (truths[IMMOBILE] == truths[BROWSING]) {
			setBrowsing();
		    } 
		    if (truths[IMMOBILE] == truths[WALKING]) {
			setWalking();
		    }
		    if (truths[IMMOBILE] == truths[DROP_DOWN]) {
			setDropDown();
		    }
		    if (truths[IMMOBILE] == truths[WINDOWSHOP_SCE]) {
			setWindowshopSce();
		    }
		    if (truths[IMMOBILE] == truths[SHOPENTER_SCE]) {
			setShopenterSce();
		    }
		    if (truths[IMMOBILE] == truths[SHOPEXIT_SCE]) {
			setShopexitSce();
		    }
		    if (truths[IMMOBILE] == truths[SHOPREENTER_SCE]) {
			setShopreenterSce();
		    }
		}	
    }

    /** 
     * Toggles the value of the immobile attribute of this SingleBox object.
     */
    public void setWalking() {
	truths[WALKING] = !truths[WALKING];
	if (truths[WALKING] == true) {
		    if (truths[WALKING] == truths[NONE_SCE]) {
			setNoneSce();
		    } 
		    if (truths[WALKING] == truths[BROWSING]) {
			setBrowsing();
		    } 
		    if (truths[WALKING] == truths[IMMOBILE]) {
			setImmobile();
		    }
		    if (truths[WALKING] == truths[DROP_DOWN]) {
			setDropDown();
		    }
		    if (truths[WALKING] == truths[WINDOWSHOP_SCE]) {
			setWindowshopSce();
		    }
		    if (truths[WALKING] == truths[SHOPENTER_SCE]) {
			setShopenterSce();
		    }
		    if (truths[WALKING] == truths[SHOPEXIT_SCE]) {
			setShopexitSce();
		    }
		    if (truths[WALKING] == truths[SHOPREENTER_SCE]) {
			setShopreenterSce();
		    }
		}	
    }

    /**
     * Toggles the value of the drop down attribute of this SingleBox object.
     */
    public void setDropDown() {
	truths[DROP_DOWN] = !truths[DROP_DOWN];
	if (truths[DROP_DOWN] == true) {
		    if (truths[DROP_DOWN] == truths[NONE_SCE]) {
			setNoneSce();
		    } 
		    if (truths[DROP_DOWN] == truths[BROWSING]) {
			setBrowsing();
		    } 
		    if (truths[DROP_DOWN] == truths[IMMOBILE]) {
			setImmobile();
		    }
		    if (truths[DROP_DOWN] == truths[WALKING]) {
			setWalking();
		    }
		    if (truths[DROP_DOWN] == truths[WINDOWSHOP_SCE]) {
			setWindowshopSce();
		    }
		    if (truths[DROP_DOWN] == truths[SHOPENTER_SCE]) {
			setShopenterSce();
		    }
		    if (truths[DROP_DOWN] == truths[SHOPEXIT_SCE]) {
			setShopexitSce();
		    }
		    if (truths[DROP_DOWN] == truths[SHOPREENTER_SCE]) {
			setShopreenterSce();
		    }
		}		
    }

    /** 
     * Toggles the value of the person immobile attribute of this SingleBox object.
     */
    public void setPersonImmobile() {
	truths[PERSON_IMMOBILE] = !truths[PERSON_IMMOBILE];
    }

    /** 
     * Sets the value of the boxDrawn attribute of this SingleBox object
     * to value.
     */
    public void setBoxDrawn(boolean value) {
	boxDrawn = value;
    }

    /** 
     * Sets the value of the selected attribute of this SingleBox object
     * to value.
     */
    public void setSelected(boolean value) {
	selected = value;
    }

    /** 
     * Sets the value of the box attribute of this SingleBox object
     * to rectangle.
     */
    public void setBox(Rectangle rectangle) {
	box = rectangle;
	setBoxDrawn(true);
    }

    /** 
     * Sets the value of the axis attribute of this SingleBox object
     * to angle.
     */
    public void setAxis(int angle) {
	axis = angle;
    }

    /** 
     * Sets the value of the axisLine attribute of this SingleBox object
     * to axisLine.
     */
    public void setLine(Line2D.Float axisLine) {
	this.axisLine = axisLine;
    }

    
    /** 
     * Sets the value of the all the values in the truths array of this SingleBox object
     * to the value of newTruths
     */
    public void setTruths(boolean[] newTruths) {
	for (int i = 0; i < truths.length; i++) {
	    truths[i] = newTruths[i];
	}
    }

    /**
     * Returns the String representing this SingleBox object.
     */
    public String toString() {
	if (boxDrawn == false) {
	    return "";
	} else {
	    
	    // box and axis representation
	    String returnString = String.valueOf((int)box.getX()+(int)(box.getWidth()/2))+ // coords are now central coords
		"\t"+String.valueOf((int)box.getY()+(int)(box.getHeight()/2))+
		"\t"+String.valueOf((int)box.getWidth())+"\t"+
		String.valueOf((int)box.getHeight()) + "\t" + axis; 
	    
	    // appearance
	    if (truths[APPEAR] == true) returnString = returnString+"\tap";
	    if (truths[DISAPPEAR] == true) returnString = returnString+"\tdi";
	    if (truths[OCCLUDED] == true) returnString = returnString+"\to";
	    
	    // movements
	    if (truths[WALK] == true) returnString = returnString+"\twk";
	    if (truths[RUN] == true) returnString = returnString+"\tr";
	    if (truths[ACTIVE] == true) returnString = returnString+"\tac";
	    if (truths[INACTIVE] == true) returnString = returnString+"\tin";
	    
	    // roles
	    if (truths[FIGHTER] == true) returnString = returnString+"\tf\t1.0";
	    if (truths[BROWSER] == true) returnString = returnString+"\tbr\t1.0";
	    if (truths[LEFT_VICTIM] == true) returnString = returnString+"\tlv\t1.0";
	    if (truths[LEAVING_GROUP] == true) returnString = returnString+"\tlg\t1.0";
	    if (truths[WALKER] == true) returnString = returnString+"\twr\t1.0";
	    if (truths[LEFT_OBJECT] == true) returnString = returnString+"\tlo\t1.0";

	    // situations
	    if (truths[MOVING_SIT] == true) returnString = returnString+"\tm\t1.0";
	    if (truths[INACTIVE_SIT] == true) returnString = returnString+"\tis\t1.0";
	    if (truths[BROWSING_SIT] == true) returnString = returnString+"\tbsi\t1.0";
	    if (truths[SHOPENTER_SIT] == true) returnString = returnString+"\tient\t1.0";
	    if (truths[SHOPEXIT_SIT] == true) returnString = returnString+"\tiext\t1.0";
	    
	    //scenarios
	    if (truths[BROWSING] == true) returnString = returnString+"\tbsc\t1.0";
	    if (truths[IMMOBILE] == true) returnString = returnString+"\tim\t1.0";
	    if (truths[WALKING] == true) returnString = returnString+"\twg\t1.0";
	    if (truths[DROP_DOWN] == true) returnString = returnString+"\tdd\t1.0";
	    if (truths[WINDOWSHOP_SCE] == true) returnString = returnString+"\tiwsp\t1.0";
	    if (truths[SHOPENTER_SCE] == true) returnString = returnString+"\tisnt\t1.0";
	    if (truths[SHOPEXIT_SCE] == true) returnString = returnString+"\tisxt\t1.0";
	    if (truths[SHOPREENTER_SCE] == true) returnString = returnString+"\tiren\t1.0";	    	    
	    // events
	    if (truths[PERSON_IMMOBILE] == true) returnString = returnString+"\tpi\t1.0";

	    return returnString;
	}
    }
 }
 

